# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'preferences_dialog.ui'
##
## Created by: Qt User Interface Compiler version 5.15.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_Preferences_Dialog(object):
    def setupUi(self, Preferences_Dialog):
        if not Preferences_Dialog.objectName():
            Preferences_Dialog.setObjectName(u"Preferences_Dialog")
        Preferences_Dialog.resize(281, 182)
        self.gridLayout = QGridLayout(Preferences_Dialog)
        self.gridLayout.setObjectName(u"gridLayout")
        self.label = QLabel(Preferences_Dialog)
        self.label.setObjectName(u"label")

        self.gridLayout.addWidget(self.label, 1, 0, 1, 1)

        self.increment_spinbox = QSpinBox(Preferences_Dialog)
        self.increment_spinbox.setObjectName(u"increment_spinbox")
        self.increment_spinbox.setMinimum(1)
        self.increment_spinbox.setMaximum(10000)

        self.gridLayout.addWidget(self.increment_spinbox, 1, 1, 1, 1)

        self.station_id_spinbox = QSpinBox(Preferences_Dialog)
        self.station_id_spinbox.setObjectName(u"station_id_spinbox")
        self.station_id_spinbox.setMinimum(0)

        self.gridLayout.addWidget(self.station_id_spinbox, 2, 1, 1, 1)

        self.label_2 = QLabel(Preferences_Dialog)
        self.label_2.setObjectName(u"label_2")

        self.gridLayout.addWidget(self.label_2, 2, 0, 1, 1)

        self.buttonBox = QDialogButtonBox(Preferences_Dialog)
        self.buttonBox.setObjectName(u"buttonBox")
        self.buttonBox.setOrientation(Qt.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.Cancel|QDialogButtonBox.Ok)

        self.gridLayout.addWidget(self.buttonBox, 3, 0, 1, 2)

        self.sn_service_combo = QComboBox(Preferences_Dialog)
        self.sn_service_combo.addItem("")
        self.sn_service_combo.addItem(u"eGauge")
        self.sn_service_combo.setObjectName(u"sn_service_combo")

        self.gridLayout.addWidget(self.sn_service_combo, 0, 1, 1, 1)

        self.label_3 = QLabel(Preferences_Dialog)
        self.label_3.setObjectName(u"label_3")

        self.gridLayout.addWidget(self.label_3, 0, 0, 1, 1)


        self.retranslateUi(Preferences_Dialog)
        self.buttonBox.accepted.connect(Preferences_Dialog.accept)
        self.buttonBox.rejected.connect(Preferences_Dialog.reject)

        QMetaObject.connectSlotsByName(Preferences_Dialog)
    # setupUi

    def retranslateUi(self, Preferences_Dialog):
        Preferences_Dialog.setWindowTitle(QCoreApplication.translate("Preferences_Dialog", u"Dialog", None))
        self.label.setText(QCoreApplication.translate("Preferences_Dialog", u"Serial-number increment", None))
#if QT_CONFIG(tooltip)
        self.station_id_spinbox.setToolTip(QCoreApplication.translate("Preferences_Dialog", u"The serial-number modulo the serial-number increment must equal the station number.", None))
#endif // QT_CONFIG(tooltip)
        self.station_id_spinbox.setPrefix("")
        self.label_2.setText(QCoreApplication.translate("Preferences_Dialog", u"Station id", None))
        self.sn_service_combo.setItemText(0, QCoreApplication.translate("Preferences_Dialog", u"none", None))

        self.label_3.setText(QCoreApplication.translate("Preferences_Dialog", u"Serial-number service to use", None))
    # retranslateUi




#
# Copyright (c) 2020-2021 eGauge Systems LLC
#       1644 Conestoga St, Suite 2
#       Boulder, CO 80301
#       voice: 720-545-9767
#       email: davidm@egauge.net
#
# All rights reserved.
#
# MIT License
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
'''Local serial number manager.

This manager works locally (no network connectivity required) and
assumes that there may be up to N different programming stations.  It
returns a sequence of serial numbers that are guaranteed to be unique,
provided the station id of each station is unique and the step value
is >= N.

'''

import os

from PySide2.QtWidgets import QMessageBox

from ctid_programmer import sn

class Manager(sn.Manager):

    def __init__(self, ui, state_directory_path):
        super().__init__(ui, os.path.join(state_directory_path,
                                          'serial_numbers.bin'))
        self.increment = 1
        self.first = 0

    def set_preferences(self, prefs):
        self._set_stepping(prefs.sn_increment, prefs.station_id)

    def activate(self, sn):
        old_sn = self.get()
        if old_sn is None:
            self._reset_serial_number(sn)
        elif sn != old_sn:
            choice = QMessageBox \
                     .question(self.ui,
                               'Serial Number Selection',
                               ('Would you like to switch to %d '
                                'as the next automatic serial number '
                                'for this product?  If you press "No", '
                                'the program will revert back to '
                                'the old sequence and use %d '
                                'as the next automatic serial number.'
                                % (sn, old_sn)),
                               QMessageBox.Yes, QMessageBox.No)
            if choice == QMessageBox.Yes:
                self._reset_serial_number(sn)

    def deactivate(self):
        pass

    def get(self):
        sn = super().get()
        if sn is None:
            sn = self.first
            super().set(sn)
        return sn

    def commit(self, sn, info, auto_serial_number):
        if auto_serial_number:
            self._increment_serial_number()
        else:
            super().set(None)

    def _set_stepping(self, increment, first):
        if increment < 1:
            raise ValueError('Increment must be greater than 0.')
        if first >= increment:
            raise ValueError('First valid serial-number must be in range '
                             '0..%d.' % (self.increment - 1))
        self.increment = increment
        self.first = first

    def _next_valid_serial_number(self, sn):
        mod = sn % self.increment
        if mod != self.first:
            # round up to first valid serial number
            sn += (self.first - mod) % self.increment
        return sn

    def _increment_serial_number(self):
        super().set(super().get() + self.increment)

    def _reset_serial_number(self, new_next_serial_number):
        next_sn = self._next_valid_serial_number(new_next_serial_number)
        super().set(next_sn)



#!/usr/bin/env python3
#
# Copyright (c) 2016-2017, 2019-2021 eGauge Systems LLC
# 	1644 Conestoga St, Suite 2
# 	Boulder, CO 80301
# 	voice: 720-545-9767
# 	email: davidm@egauge.net
#
#  All rights reserved.
#
# MIT License
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
import argparse
import logging
import os
import re
import subprocess
import sys
import tempfile

import importlib_resources
import pexpect
import pkg_resources

from pexpect import fdpexpect

from PySide2.QtCore import Qt
from PySide2.QtWidgets import QAction, QApplication, \
    QDialog, QDialogButtonBox, QMainWindow, QMessageBox, QListWidgetItem

import egauge.ctid as ctid

from egauge.pyside2 import terminal

from ctid_programmer import preferences
from ctid_programmer import sensor_params
from ctid_programmer import sn
from ctid_programmer import sn_egauge
from ctid_programmer import sn_local
from ctid_programmer import template

from ctid_programmer.gui.main_window import Ui_MainWindow
from ctid_programmer.gui.template_dialog import Ui_Template_Dialog
from ctid_programmer.gui.preferences_dialog import Ui_Preferences_Dialog
from ctid_programmer.gui.param_ct import Ui_Param_CT
from ctid_programmer.gui.param_linear import Ui_Param_Linear
from ctid_programmer.gui.param_temp import Ui_Param_Temp
from ctid_programmer.gui.param_ntc import Ui_Param_NTC
from ctid_programmer.gui.param_pulse import Ui_Param_Pulse


PACKAGE_NAME = 'CTid-programmer'
PACKAGE_VERSION = pkg_resources.get_distribution('CTid-programmer').version
VERSION_INFO = f'''{PACKAGE_NAME} {PACKAGE_VERSION}
Copyright (c) 2018-2022 eGauge Systems LLC
License MIT: <https://opensource.org/licenses/MIT>.
This is free software: you are free to change and redistribute it.

Written by Alexandra Kaufhold and David Mosberger.'''

PATH_AVRDUDE = 'avrdude'
PATH_CTID_ENCODER = 'ctid-encoder'

PATH_STATE_DIR = os.path.join(os.getenv('HOME'), '.CTid')

SENSOR_LONG_NAME = {
    'AC': 'AC Current Sensor',
    'DC': 'DC Current Sensor',
    'RC': 'Rogowski Coil Sensor',
    'linear': 'Linear Sensor',
    'temp': 'Linear Temperature Sensor',
    'NTC': 'NTC Thermistor Sensor',
    'pulse': 'Pulse Sensor'
}

# Filename of template to use for each sensor-type:
CODE_TEMPLATE = {
    'AC': 'ac.hex',
    'RC': 'ac.hex'
}

CHIP_ID_TO_NAME = {
    0x1e9008: ('t9', 'ATtiny9'),
    0x1e9003: ('t10', 'ATtiny10')
}

class Command_Processor:
    def __init__(self, argv, pattern_list=None, logfile=None,
                 stdout=None):
        self.pattern_list = pattern_list + [pexpect.EOF, pexpect.TIMEOUT]
        self.error = None
        self.exit_status = None
        self.pipe = None
        self.prog = None

        if stdout is not None:
            self.pipe = subprocess.Popen(argv, stdout=stdout,
                                         stdin=subprocess.DEVNULL,
                                         stderr=subprocess.PIPE)

        try:

            if self.pipe is None:
                self.prog = pexpect.spawn(argv[0], argv[1:],
                                          encoding='utf-8',
                                          codec_errors='replace',
                                          logfile=logfile)
            else:
                self.prog = fdpexpect.fdspawn(self.pipe.stderr,
                                              encoding='utf-8',
                                              codec_errors='replace',
                                              logfile=logfile)
        except pexpect.ExceptionPexpect:
            self.error = 'Failed to start command: %s' % sys.exc_info()[1]
            return

    def __iter__(self):
        return self

    def __next__(self):
        while self.prog is not None:
            got = self.prog.expect(self.pattern_list, timeout=0.1)
            if got < len(self.pattern_list) - 2:
                return (got, self.prog.match)
            if got == len(self.pattern_list) - 2:
                if self.pipe is None:
                    self.prog.close()
                    self.exit_status = self.prog.exitstatus
                else:
                    self.pipe.wait()
                    self.exit_status = self.pipe.returncode
                break   # EOF: done
            if got == len(self.pattern_list) - 1:
                pass    # timeout; process events and then try again...
            QApplication.processEvents()
        raise StopIteration

    def stop(self):
        if self.error is None:
            self.error = 'program interrupted by user'
        self.prog.close()
        self.prog = None

class Template_List_Dialog(QDialog, Ui_Template_Dialog):
    def __init__(self, parent):
        super().__init__(parent)
        self.parent = parent
        self.setupUi(self)
        self.listWidget.itemChanged.connect(self.item_changed)
        self.listWidget.itemDoubleClicked.connect(self.item_double_clicked)
        self.operation = None

        action = QAction('Rename', self)
        action.triggered.connect(self.rename_selected_template)
        self.listWidget.addAction(action)

        action = QAction('Delete', self)
        action.triggered.connect(self.delete_selected_template)
        self.listWidget.addAction(action)

        template_names = []
        for name, _ in self.parent.template_manager.items():
            template_names.append(name)
        template_names.sort()
        for name in template_names:
            self.add_template(name)

    def add_template(self, name):
        item = QListWidgetItem(name, self.listWidget)
        item.setFlags(item.flags() | Qt.ItemIsEditable)
        # save a copy of the original name as user data for renames:
        item.setData(Qt.UserRole, name)

    def rename_selected_template(self, is_checked):
        #pylint: disable=unused-argument
        self.listWidget.editItem(self.listWidget.currentItem())

    def delete_selected_template(self, is_checked):
        #pylint: disable=unused-argument
        item = self.listWidget.currentItem()
        self.parent.template_manager.remove(item.text())

        idx = self.listWidget.row(item)
        self.listWidget.takeItem(idx)

    def item_double_clicked(self):
        if self.operation == 'save':
            self.lineEdit.setText(self.listWidget.currentItem().text())
        self.accept()

    def item_changed(self):
        item = self.listWidget.currentItem()
        if item is None:
            return
        new_name = item.text()
        old_name = item.data(Qt.UserRole)
        if new_name != old_name:
            template = self.parent.template_manager.load(old_name)
            self.parent.template_manager.save(template, new_name)
            self.parent.template_manager.remove(old_name)

    def accept(self):
        if self.operation == 'save':
            new_name = self.lineEdit.text().strip().lstrip()
            if not new_name:
                QMessageBox.warning(self, 'Template Name Missing',
                                    'Please enter a template name.',
                                    QMessageBox.Ok)
                self.lineEdit.setFocus()
                self.lineEdit.selectAll()
                return
            if new_name in dict(self.parent.template_manager.items()):
                choice = QMessageBox.question(self,
                                              'Template Name Exists',
                                              'A template named `%s\' '
                                              'exists already.  '
                                              'Would you like to replace '
                                              'that template?' % new_name,
                                              QMessageBox.Yes,
                                              QMessageBox.Cancel)
                if choice == QMessageBox.Cancel:
                    return	# let user correct name
            else:
                self.add_template(new_name)
            template = self.parent.get_template()
            self.parent.template_manager.save(template, new_name)
        else:
            item = self.listWidget.currentItem()
            name = item.text()
            template = self.parent.template_manager.load(name)
            self.parent.template_activate(template)
            self.parent.log('Template `%s\' loaded.' % name)

        super().accept()

class Preferences_Editor(QDialog, Ui_Preferences_Dialog):
    def __init__(self, parent):
        super().__init__(parent)
        self.parent = parent
        self.setupUi(self)

    def exec_(self):
        self.sn_service_combo.currentIndexChanged.connect(
            self.sn_service_changed)
        self.sn_service_combo.setCurrentIndex(0)
        selected_service = self.parent.preferences.sn_service
        if selected_service is not None:
            for idx in range(1, self.sn_service_combo.count()):
                service_name = self.sn_service_combo.itemText(idx)
                if service_name == selected_service:
                    self.sn_service_combo.setCurrentIndex(idx)
                    break
        self.increment_spinbox.setValue(self.parent.preferences.sn_increment)
        self.station_id_spinbox.setValue(self.parent.preferences.station_id)
        return super().exec_()

    def accept(self):
        increment = self.increment_spinbox.value()
        station_id = self.station_id_spinbox.value()

        if station_id >= increment:
            QMessageBox.critical(self, 'Invalid Station Number',
                                 'Station id (%d) must be smaller than '
                                 'the serial-number increment (%d).'
                                 % (station_id, increment), QMessageBox.Ok)
            return
        super().accept()
        prefs = self.parent.preferences
        if self.sn_service_combo.currentIndex() == 0:
            prefs.sn_service = None
        else:
            prefs.sn_service = self.sn_service_combo.currentText()
        prefs.sn_increment = increment
        prefs.station_id = station_id
        prefs.save()
        self.parent.prefs_changed()

    def sn_service_changed(self):
        enable = (self.sn_service_combo.currentIndex() == 0)
        self.increment_spinbox.setEnabled(enable)
        self.station_id_spinbox.setEnabled(enable)

class UI(QMainWindow, Ui_MainWindow):
    def __init__(self):
        QMainWindow.__init__(self)
        self.setupUi(window)

        self.params = {}

        ct_params = sensor_params.CT(Ui_Param_CT(), self.param_group,
                                     'CT Parameters')
        self.params['AC'] = self.params['DC'] = self.params['RC'] = ct_params
        self.params['linear'] = sensor_params.Linear(Ui_Param_Linear(),
                                                     self.param_group,
                                                     'Linear Parameters')
        self.params['temp'] = sensor_params.Temp(Ui_Param_Temp(),
                                                 self.param_group,
                                                 'Temperature Parameters')
        self.params['NTC'] = sensor_params.NTC(Ui_Param_NTC(), self.param_group,
                                               'NTC Thermistor Parameters')
        self.params['pulse'] = sensor_params.Pulse(Ui_Param_Pulse(),
                                                   self.param_group,
                                                   'Pulse Parameters')

        self.current_params = self.params['AC']
        self.current_params.activate()

        self.auto_serial_checkbox.setChecked(True)
        self.serial_spinbox.setEnabled(False)
        self.console = terminal.Terminal(self.plainTextEdit)
        self.add_mfgs()
        self.busy = False
        self.cmd = None
        self.sn_service = None
        self.mfg_id = None
        self.model = None

        self.console.write('Welcome to the CTid® Programmer!\n\n'
                           'You can use this tool to program '
                           'the microcontroller of a CTid board (PCB).\n\n'
                           'Please start by filling out the form on the left. '
                           'Then attach the programming cable to the '
                           'CTid board, and click "Program" to write the '
                           'information to the microcontroller.\n')
        self.welcome_msg = True	# we're displaying welcome message

        if not os.path.isdir(PATH_STATE_DIR):
            os.mkdir(PATH_STATE_DIR)
        self.preferences = preferences.Manager(PATH_STATE_DIR)
        self.sn = None
        self.template_manager = template.Manager(PATH_STATE_DIR)

        for st in ctid.SENSOR_TYPE_NAME:
            name = SENSOR_LONG_NAME[st] if st in SENSOR_LONG_NAME else st
            self.sensor_type_combo.addItem(name)

        self.action_About.triggered.connect(self.about)
        self.sensor_type_combo.currentIndexChanged.connect(
            self.sensor_type_changed)
        self.mfg_combo.currentIndexChanged.connect(self.product_changed)
        self.model_lineEdit.editingFinished.connect(self.product_changed)
        self.auto_serial_checkbox.stateChanged.connect(self.auto_serial_changed)
        self.program_btn.clicked.connect(self.program_or_cancel)
        self.read_btn.clicked.connect(self.read)
        self.reprogram_after_cal_btn.clicked.connect(self.reprogram_after_cal)
        self.load_template_btn.clicked.connect(self.template_load)
        self.save_template_btn.clicked.connect(self.template_save)
        self.save_template_btn.setVisible(self.template_manager.may_save())

        self.template_list_dialog = Template_List_Dialog(self)

        self.preferences_editor = Preferences_Editor(self)
        self.actionPreferences.triggered.connect(self.preferences_editor.exec)

        self.prefs_changed()

    def about(self):
        QMessageBox.about(self, 'About', VERSION_INFO)

    def log(self, msg):
        if self.welcome_msg:
            self.plainTextEdit.clear()
            self.welcome_msg = False
        self.console.write(msg + '\n')

    def template_activate(self, template):
        '''Load values from template, except never load the serial-number.'''
        try:
            if 'model' in template:
                self.model_lineEdit.setText(template['model'])

            if 'mfg' in template:
                for idx in range(self.mfg_combo.count()):
                    if self.mfg_combo.itemData(idx) == template['mfg']:
                        # this may trigger call to product_changed() so
                        # model_lineEdit() must have been updated already...
                        self.mfg_combo.setCurrentIndex(idx)

            self.product_changed()

            # 'voltage' sensor type is now called 'linear':
            if template['sensor_type'] == 'voltage':
                template['sensor_type'] = 'linear'
                template['unit'] = 0

            self.sensor_type_combo.setCurrentIndex(0)
            for idx, code in enumerate(ctid.SENSOR_TYPE_NAME):
                if code == template['sensor_type']:
                    self.sensor_type_combo.setCurrentIndex(idx)
                    break

            self.r_source_spinbox.setValue(template['r_source'])
            self.r_load_spinbox.setValue(template['r_load'])

            if self.current_params is not None:
                self.current_params.load(template)
        except KeyError as key:
            QMessageBox.warning(self, 'Warning',
                                'Template is missing parameter %s.'
                                % (key), QMessageBox.Ok)

    def template_load(self):
        self.template_list_dialog.operation = 'load'
        self.template_list_dialog.template_name_frame.hide()
        self.template_list_dialog.buttonBox.setStandardButtons(
            QDialogButtonBox.Open|QDialogButtonBox.Cancel)
        self.template_list_dialog.exec()

    def template_save(self):
        self.template_list_dialog.operation = 'save'
        self.template_list_dialog.template_name_frame.show()
        self.template_list_dialog.buttonBox.setStandardButtons(
            QDialogButtonBox.Save|QDialogButtonBox.Cancel)
        self.template_list_dialog.exec()

    def get_mfg_id(self):
        mfg_idx = self.mfg_combo.currentIndex()
        if mfg_idx == 0:
            return None
        return self.mfg_combo.itemData(mfg_idx)

    def get_model(self):
        model = self.model_lineEdit.text().strip().lstrip()
        if not model:
            return None
        return model

    def update_sn(self):
        if not self.auto_serial_checkbox.isChecked():
            return

        if self.get_mfg_id() is None or self.get_model() is None:
            return

        next_sn = None
        try:
            next_sn = self.sn.get()
            if next_sn is None:
                QMessageBox.warning(self, 'Serial Number Unavailable',
                                    'Serial number service failed to return '
                                    'a serial number.  Reverting to '
                                    'manual serial numbers.',
                                    QMessageBox.Ok)
        except sn.SpaceExhausted:
            QMessageBox.warning(self, 'Serial Number Space Exhausted',
                                'All available serial-numbers have been used '
                                'for \"%s %s\".  Please use a different '
                                'manufacturer and/or model name.' \
                                % (ctid.mfg_short_name(self.mfg_id),
                                   self.model), QMessageBox.Ok)

        if next_sn is None:
            self.auto_serial_checkbox.setChecked(False)
            self.serial_spinbox.setEnabled(True)
            self.serial_spinbox.setValue(0)
            return

        self.serial_spinbox.setValue(next_sn)

    def sensor_type_changed(self):
        st = ctid.SENSOR_TYPE_NAME[self.sensor_type_combo.currentIndex()]
        new_params = self.params[st] if st in self.params else None

        if new_params == self.current_params:
            return	# no change

        if self.current_params is not None:
            self.current_params.deactivate()
        self.current_params = new_params

        if self.current_params is not None:
            self.current_params.activate()

    def product_changed(self):
        mfg_id = self.get_mfg_id()
        model = self.get_model()

        log.debug('product_changed: mfg_id=%s model=%s', mfg_id, model)

        if mfg_id is None or model is None:
            return

        if self.mfg_id == mfg_id and self.model == model:
            return	# no change
        self.mfg_id = mfg_id
        self.model = model

        if self.sn.set_product(mfg_id, model):
            self.update_sn()
        else:
            self.auto_serial_checkbox.setChecked(False)
            self.serial_spinbox.setValue(0)

    def auto_serial_changed(self):
        auto_serial = self.auto_serial_checkbox.isChecked()
        if auto_serial:
            latest_sn = self.serial_spinbox.value()
            self.sn.activate(latest_sn)
            self.update_sn()
        else:
            self.sn.deactivate()
        self.serial_spinbox.setEnabled(not auto_serial)

    def add_mfgs(self):
        '''Add companies per their table/manufacturer number as stated by CTid
        Spec Sheet.

        '''
        for mfg_id, name in ctid.MFG_ID.items():
            self.mfg_combo.addItem(name, mfg_id)

    def set_input_enabled(self, enabled):
        self.chip_combo.setEnabled(enabled)
        self.mfg_combo.setEnabled(enabled)
        self.sensor_type_combo.setEnabled(enabled)
        self.model_lineEdit.setEnabled(enabled)
        self.auto_serial_checkbox.setEnabled(enabled)
        self.r_source_spinbox.setEnabled(enabled)
        self.r_load_spinbox.setEnabled(enabled)

        serial_enabled = enabled and not self.auto_serial_checkbox.isChecked()
        self.serial_spinbox.setEnabled(serial_enabled)

        if self.current_params is not None:
            self.current_params.set_input_enabled(enabled)

        self.load_template_btn.setEnabled(enabled)
        self.save_template_btn.setEnabled(enabled)
        self.read_btn.setEnabled(enabled)
        self.reprogram_after_cal_btn.setEnabled(enabled)

    def cmd_start(self, argv, pattern_list, logfile=None, stdout=None):
        self.cmd = Command_Processor(argv, pattern_list, logfile=logfile,
                                     stdout=stdout)

    def cmd_done(self):
        if self.cmd.error is not None:
            self.log('Command failed: %s' % self.cmd.error)

    def get_template(self):
        '''Get form-data as a dictionary.  No validation is performed beyond
        the constraints imposed by the user-interface controls.

        '''
        template = {}

        mfg_id = self.get_mfg_id()
        if mfg_id is not None:
            template['mfg'] = mfg_id

        model = self.get_model()
        if model is not None:
            template['model'] = model

        template['sn'] = self.serial_spinbox.value()
        st_idx = self.sensor_type_combo.currentIndex()
        template['sensor_type'] = ctid.SENSOR_TYPE_NAME[st_idx]
        template['r_source'] = self.r_source_spinbox.value()
        template['r_load'] = self.r_load_spinbox.value()

        if self.current_params is not None:
            self.current_params.save(template)
        return template

    def validate_form(self):
        '''Validate form data and return cleaned data as a dictionary.'''
        cleaned_data = self.get_template()

        if 'mfg' not in cleaned_data:
            QMessageBox.warning(self, 'Manufacturer Missing',
                                'Please select manufacturer.', QMessageBox.Ok)
            self.mfg_combo.setFocus()
            return None

        if 'model' not in cleaned_data:
            QMessageBox.warning(self, 'Model Name Missing',
                                'Please enter model name.', QMessageBox.Ok)
            self.model_lineEdit.setFocus()
            return None

        utf8_model = cleaned_data['model'].encode('utf-8')
        if len(utf8_model) > 8:
            QMessageBox.warning(self, 'Model Name Too Long',
                                'Model name is %d bytes long in '
                                'UTF-8 encoding.  '
                                'Please limit name to 8 bytes in length.'
                                % len(utf8_model), QMessageBox.Ok)
            self.model_lineEdit.setFocus()
            return None
        return cleaned_data

    def show_cmd_error(self, cmd, title, msg):
        if cmd.exit_status is not None:
            msg += '  Exit status %d.' % cmd.exit_status
        QMessageBox.warning(self, title, msg, QMessageBox.Ok)

    def create_hexfile(self, cleaned_data, params):
        st = cleaned_data['sensor_type']
        argv = [PATH_CTID_ENCODER,
                '-M', '%d' % cleaned_data['mfg'],
                '-S', st,
                '-m', cleaned_data['model'],
                '-n', '%d' % cleaned_data['sn'],
                '-l', '%d' % cleaned_data['r_load'],
                '-r', '%d' % cleaned_data['r_source']]

        if params is not None:
            argv += params.encoder_argv(cleaned_data)

        output = tempfile.mkstemp(suffix='.hex', prefix='CTid-')

        template_filename = CODE_TEMPLATE[st] if st in CODE_TEMPLATE \
                            else 'powered.hex'

        ref = (importlib_resources.files('ctid_programmer')
               / 'resources' / 'code' / template_filename)
        with importlib_resources.as_file(ref) as code_template_path:
            argv.append(str(code_template_path))

            log.debug('create_hexfile: argv=%s', argv)

            self.cmd_start(argv, [], logfile=self.console,
                           stdout=output[0])
            # consume command's output by iterating over it:
            for _ in self.cmd:
                pass
            self.cmd_done()

        if self.cmd.exit_status != 0:
            self.show_cmd_error(self.cmd, 'Command Failed',
                                'Failed to create hex file.')
            os.close(output[0])
            os.remove(output[1])
            return None

        # now that we have created the actual hexfile, it's safe to close
        # the original (empty) file created by mkstemp():
        os.close(output[0])
        return output[1]

    def detect_chip_type(self):
        self.cmd_start([PATH_AVRDUDE, '-ctc2030', '-pt9', '-nq'],
                       [r'Device signature = 0x([0-9a-f]+)'])
        chip_id = None
        for _, match in self.cmd:
            chip_id = int(match.group(1), base=16)
        self.cmd_done()
        if chip_id is None:
            QMessageBox.warning(self, 'No Microcontroller Detected',
                                'No microcontroller detected.  '
                                'Please confirm programming cable is '
                                'properly attached.',
                                QMessageBox.Ok)
            return None
        if chip_id not in CHIP_ID_TO_NAME:
            QMessageBox.warning(self, 'Unknown Microcontroller',
                                'Unknown microcontroller chip type 0x%x.'
                                % chip_id, QMessageBox.Ok)
            return None
        chip = CHIP_ID_TO_NAME[chip_id]
        self.log('Detected %s chip.' % chip[1])
        return chip[0]

    def write_flash(self, chip_type, hexfile, cleaned_data, auto_serial_number):
        '''Write HEXFILE to the flash to a chip of type CHIP_TYPE.  The
        HEXFILE contains the data passed in CLEANED_DATA.
        AUTO_SERILA_NUMBER must be True if the serial numbers are
        being assigned automatically, False otherwise.

        Returns True on success, False on any failure.
        '''
        self.cmd_start([PATH_AVRDUDE, '-ctc2030', '-p%s' % chip_type,
                        '-Uflash:w:%s' % hexfile], [], logfile=self.console)
        for _ in self.cmd:
            pass	# consume output until program is done...
        self.cmd_done()
        if self.cmd.exit_status != 0:
            self.show_cmd_error(self.cmd, 'Programming Failed',
                                'Failed to write the microcontroller flash.')
            return False

        self.log('Success: CTid board has been programmed with '
                 'serial number %d.' % cleaned_data['sn'])
        self.sn.commit(cleaned_data['sn'], cleaned_data, auto_serial_number)
        return True

    def read_template_from_flash(self, chip_type):
        temp = tempfile.TemporaryFile(suffix='.bin', prefix='CTid-')
        self.cmd_start([PATH_AVRDUDE, '-ctc2030', '-p%s' % chip_type,
                        '-Uflash:r:-:r'], [], logfile=self.console,
                       stdout=temp)
        for _ in self.cmd:
            pass	# consume output until program is done...
        self.cmd_done()

        if self.cmd.exit_status != 0:
            temp.close()
            self.show_cmd_error(self.cmd, 'Read Failed',
                                'Failed to read the microcontroller flash.')
            return None

        temp.seek(0)
        flash = temp.read()
        temp.close()

        if len(flash) < 0x3e1:
            if not flash:
                QMessageBox.critical(self, 'Read Failed',
                                     'Microcontroller flash is empty.',
                                     QMessageBox.Ok)
            else:
                QMessageBox.critical(self, 'Read Failed',
                                     'Read only %u bytes from '
                                     'microcontroller flash.'
                                     % len(flash), QMessageBox.Ok)
            return None
        self.log('Success: CTid board has been read.')

        CTid_table_addr = 0x3c0		# table goes in last 64 bytes
        length = flash[CTid_table_addr]
        table = flash[CTid_table_addr + 1:CTid_table_addr + 1 + length]
        table = ctid.Table(ctid.unstuff(table))
        template = self.ctid_table_to_template(table)

        if template is not None:
            m = re.match(r'(.*)-[^-]+$', template['model'])
            if m is not None:
                # strip suffix off model-name:
                template['model'] = m.group(1)
        return template

    def reprogram_with_cal_params(self, chip_type):
        template = self.read_template_from_flash(chip_type)
        if template is None:
            return

        sn = template['sn']
        mfg = template['mfg']
        model = template['model']

        saved_product = self.sn.product
        self.sn.set_product(mfg, model)
        cal_params = self.sn.get_cal_data(sn)
        if cal_params is None:
            self.sn.restore_product(saved_product)
            QMessageBox.critical(self, 'No Calibration Data Found',
                                 'No calibration data was found for '
                                 'product %s with serial number %s.' \
                                 % (self.sn.product, sn), QMessageBox.Ok)
            return

        msg = ''
        for key, value in cal_params.items():
            msg += '\t%16s: %s\n' % (key, value)

        # merge the calibrated parameters with existing template:
        for name, value in cal_params.items():
            template[name] = value
        model = template['model']

        self.console.write('\n\nFound calibration data for %s %s #%d:\n%s\n' %
                           (ctid.mfg_short_name(mfg), model, sn, msg))

        hexfile = self.create_hexfile(template,
                                      self.params[template['sensor_type']])
        if hexfile is None:
            self.sn.restore_product(saved_product)
            self.mark_idle()
            return
        self.write_flash(chip_type, hexfile, template,
                         auto_serial_number=False)
        os.remove(hexfile)
        self.mark_idle()

        self.sn.restore_product(saved_product)

    def read_flash(self, chip_type):
        template = self.read_template_from_flash(chip_type)
        if template is None:
            return
        self.template_activate(template)
        self.auto_serial_checkbox.setChecked(False)
        self.serial_spinbox.setValue(template['sn'])

    def ctid_table_to_template(self, table):
        sensor_type = ctid.SENSOR_TYPE_NAME[table.sensor_type]
        template = {}
        template['mfg'] = table.mfg_id
        template['model'] = table.model
        template['sn'] = table.serial_number
        template['sensor_type'] = sensor_type
        template['r_source'] = table.r_source
        template['r_load'] = table.r_load
        self.params[sensor_type].table_to_template(table, template)
        return template

    def mark_busy(self):
        self.plainTextEdit.clear()
        self.busy = True
        self.set_input_enabled(False)
        self.program_btn.setText('Cancel')

    def mark_idle(self):
        self.busy = False
        self.program_btn.setText('Program')
        self.set_input_enabled(True)

    def chip_type(self):
        if self.chip_combo.currentIndex() == 0:
            # auto-detect chip
            chip_type = self.detect_chip_type()
        elif self.chip_combo.currentIndex() == 1:
            chip_type = 't9'
        else:
            chip_type = 't10'
        return chip_type

    def program_or_cancel(self):
        '''Program the microcontroller with the info specified in the form.
        This consists of two steps: (1) creating a hex file with the
        info encoded and (2) writing the file to the microcontroller.

        '''
        if self.busy:
            if self.cmd is not None:
                self.cmd.stop()	# cancel the running command
                self.cmd_done()
            self.mark_idle()
            return

        cleaned_data = self.validate_form()
        if cleaned_data is None:
            return

        self.mark_busy()

        hexfile = self.create_hexfile(cleaned_data, self.current_params)
        if hexfile is None:
            self.mark_idle()
            return

        chip_type = self.chip_type()
        if chip_type is not None:
            auto_serial_number = self.auto_serial_checkbox.isChecked()
            if self.write_flash(chip_type, hexfile, cleaned_data,
                                auto_serial_number):
                self.update_sn()

        os.remove(hexfile)
        self.mark_idle()

    def read(self):
        '''Reads CTid parameters from the flash.'''
        self.mark_busy()

        chip_type = self.chip_type()
        if chip_type is not None:
            self.read_flash(chip_type)

        self.mark_idle()

    def reprogram_after_cal(self):
        '''Reads CTid parameters from the flash, lookup calibration parameters
        for the read serial-number and, if that exists, reprogram with
        those parameters.

        '''
        self.mark_busy()

        chip_type = self.chip_type()
        if chip_type is not None:
            self.reprogram_with_cal_params(chip_type)

        self.mark_idle()

    def prefs_changed(self):
        prefs = self.preferences

        if self.sn is None or self.sn_service != prefs.sn_service:
            self.sn_service = prefs.sn_service
            if prefs.sn_service == 'eGauge':
                self.sn = sn_egauge.Manager(self, PATH_STATE_DIR)
            else:
                if prefs.sn_service is not None:
                    QMessageBox.critical(self, 'Unknown Serial Number Service',
                                         'Serial number service %s is '
                                         'unknown.  Reverting to locally '
                                         'managed serial numbers.' \
                                         % prefs.sn_service)
                    self.sn_service = None
                self.sn = sn_local.Manager(self, PATH_STATE_DIR)
            if self.mfg_id is not None and self.model is not None:
                self.sn.set_product(self.mfg_id, self.model)
            self.reprogram_after_cal_btn.setVisible(
                self.sn.has_calibration_data())
        self.sn.set_preferences(prefs)
        self.update_sn()

parser = argparse.ArgumentParser(description='CTid GUI programmer.')
parser.add_argument('-F', '--full-screen', action='store_true',
                    help='Start application in full-screen mode.')
parser.add_argument('-d', '--debug', action='store_const', const=logging.DEBUG,
                    dest='log_level', help='Show debug output.')
parser.add_argument('-v', '--version', action='store_true',
                    help='Output version information and exit.')
args = parser.parse_args()

if args.version:
    print(VERSION_INFO)
    sys.exit(0)

log_level = logging.ERROR if args.log_level is None else args.log_level
logging.basicConfig()
log = logging.getLogger()	# get the root logger
log.setLevel(log_level)		# sets default logging for all child loggers

app = QApplication(sys.argv)
window = QMainWindow()
ui = UI()

if args.full_screen:
    window.showMaximized()
else:
    window.show()
sys.exit(app.exec_())



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'param_temp.ui'
##
## Created by: Qt User Interface Compiler version 5.15.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_Param_Temp(object):
    def setupUi(self, Param_Temp):
        if not Param_Temp.objectName():
            Param_Temp.setObjectName(u"Param_Temp")
        Param_Temp.resize(318, 95)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Param_Temp.sizePolicy().hasHeightForWidth())
        Param_Temp.setSizePolicy(sizePolicy)
        self.horizontalLayout = QHBoxLayout(Param_Temp)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalLayout.setContentsMargins(-1, 24, -1, -1)
        self.gridLayout = QGridLayout()
        self.gridLayout.setObjectName(u"gridLayout")
        self.offset_spinbox = QDoubleSpinBox(Param_Temp)
        self.offset_spinbox.setObjectName(u"offset_spinbox")
        self.offset_spinbox.setDecimals(6)
        self.offset_spinbox.setMinimum(-1000000000.000000000000000)
        self.offset_spinbox.setMaximum(1000000000.000000000000000)

        self.gridLayout.addWidget(self.offset_spinbox, 1, 1, 1, 1)

        self.label_5 = QLabel(Param_Temp)
        self.label_5.setObjectName(u"label_5")
        font = QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_5.setFont(font)

        self.gridLayout.addWidget(self.label_5, 1, 0, 1, 1)

        self.label_9 = QLabel(Param_Temp)
        self.label_9.setObjectName(u"label_9")
        self.label_9.setFont(font)

        self.gridLayout.addWidget(self.label_9, 0, 0, 1, 1)

        self.scale_spinbox = QDoubleSpinBox(Param_Temp)
        self.scale_spinbox.setObjectName(u"scale_spinbox")
        self.scale_spinbox.setDecimals(9)
        self.scale_spinbox.setMinimum(-1000000000.000000000000000)
        self.scale_spinbox.setMaximum(1000000000.000000000000000)

        self.gridLayout.addWidget(self.scale_spinbox, 0, 1, 1, 1)

        self.gridLayout.setColumnStretch(0, 1)

        self.horizontalLayout.addLayout(self.gridLayout)

        QWidget.setTabOrder(self.scale_spinbox, self.offset_spinbox)

        self.retranslateUi(Param_Temp)

        QMetaObject.connectSlotsByName(Param_Temp)
    # setupUi

    def retranslateUi(self, Param_Temp):
        self.offset_spinbox.setSuffix(QCoreApplication.translate("Param_Temp", u" \u00b0C", None))
        self.label_5.setText(QCoreApplication.translate("Param_Temp", u"Offset", None))
        self.label_9.setText(QCoreApplication.translate("Param_Temp", u"Scale", None))
        self.scale_spinbox.setSuffix(QCoreApplication.translate("Param_Temp", u" \u00b0C/V", None))
        pass
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'param_ct.ui'
##
## Created by: Qt User Interface Compiler version 5.15.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_Param_CT(object):
    def setupUi(self, Param_CT):
        if not Param_CT.objectName():
            Param_CT.setObjectName(u"Param_CT")
        Param_CT.resize(387, 444)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Param_CT.sizePolicy().hasHeightForWidth())
        Param_CT.setSizePolicy(sizePolicy)
        self.gridLayout = QGridLayout(Param_CT)
        self.gridLayout.setObjectName(u"gridLayout")
        self.gridLayout.setContentsMargins(-1, 24, -1, -1)
        self.label_9 = QLabel(Param_CT)
        self.label_9.setObjectName(u"label_9")
        font = QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_9.setFont(font)

        self.gridLayout.addWidget(self.label_9, 0, 0, 1, 1)

        self.current_spinbox = QDoubleSpinBox(Param_CT)
        self.current_spinbox.setObjectName(u"current_spinbox")
        self.current_spinbox.setDecimals(1)
        self.current_spinbox.setMaximum(6553.500000000000000)

        self.gridLayout.addWidget(self.current_spinbox, 0, 1, 1, 1)

        self.label_5 = QLabel(Param_CT)
        self.label_5.setObjectName(u"label_5")
        self.label_5.setFont(font)

        self.gridLayout.addWidget(self.label_5, 1, 0, 1, 1)

        self.size_spinbox = QDoubleSpinBox(Param_CT)
        self.size_spinbox.setObjectName(u"size_spinbox")
        self.size_spinbox.setDecimals(1)
        self.size_spinbox.setMaximum(6553.500000000000000)

        self.gridLayout.addWidget(self.size_spinbox, 1, 1, 1, 1)

        self.label_10 = QLabel(Param_CT)
        self.label_10.setObjectName(u"label_10")
        self.label_10.setFont(font)

        self.gridLayout.addWidget(self.label_10, 2, 0, 1, 1)

        self.output_voltage_spinbox = QDoubleSpinBox(Param_CT)
        self.output_voltage_spinbox.setObjectName(u"output_voltage_spinbox")
        self.output_voltage_spinbox.setDecimals(5)
        self.output_voltage_spinbox.setMaximum(0.655350000000000)
        self.output_voltage_spinbox.setSingleStep(0.001000000000000)

        self.gridLayout.addWidget(self.output_voltage_spinbox, 2, 1, 1, 1)

        self.label_23 = QLabel(Param_CT)
        self.label_23.setObjectName(u"label_23")
        self.label_23.setFont(font)

        self.gridLayout.addWidget(self.label_23, 3, 0, 1, 1)

        self.phase_spinbox = QDoubleSpinBox(Param_CT)
        self.phase_spinbox.setObjectName(u"phase_spinbox")
        self.phase_spinbox.setMinimum(-20.480000000000000)
        self.phase_spinbox.setMaximum(20.469999999999999)

        self.gridLayout.addWidget(self.phase_spinbox, 3, 1, 1, 1)

        self.label_11 = QLabel(Param_CT)
        self.label_11.setObjectName(u"label_11")
        self.label_11.setFont(font)

        self.gridLayout.addWidget(self.label_11, 4, 0, 1, 1)

        self.voltage_temp_coeff_spinbox = QDoubleSpinBox(Param_CT)
        self.voltage_temp_coeff_spinbox.setObjectName(u"voltage_temp_coeff_spinbox")
        self.voltage_temp_coeff_spinbox.setDecimals(0)
        self.voltage_temp_coeff_spinbox.setMinimum(-640.000000000000000)
        self.voltage_temp_coeff_spinbox.setMaximum(635.000000000000000)
        self.voltage_temp_coeff_spinbox.setSingleStep(5.000000000000000)

        self.gridLayout.addWidget(self.voltage_temp_coeff_spinbox, 4, 1, 1, 1)

        self.label_12 = QLabel(Param_CT)
        self.label_12.setObjectName(u"label_12")
        self.label_12.setFont(font)

        self.gridLayout.addWidget(self.label_12, 5, 0, 1, 1)

        self.phase_temp_coeff_spinbox = QDoubleSpinBox(Param_CT)
        self.phase_temp_coeff_spinbox.setObjectName(u"phase_temp_coeff_spinbox")
        self.phase_temp_coeff_spinbox.setDecimals(1)
        self.phase_temp_coeff_spinbox.setMinimum(-64.000000000000000)
        self.phase_temp_coeff_spinbox.setMaximum(63.500000000000000)
        self.phase_temp_coeff_spinbox.setSingleStep(0.500000000000000)

        self.gridLayout.addWidget(self.phase_temp_coeff_spinbox, 5, 1, 1, 1)

        self.label_21 = QLabel(Param_CT)
        self.label_21.setObjectName(u"label_21")
        self.label_21.setFont(font)

        self.gridLayout.addWidget(self.label_21, 6, 0, 1, 1)

        self.bias_voltage_spinbox = QDoubleSpinBox(Param_CT)
        self.bias_voltage_spinbox.setObjectName(u"bias_voltage_spinbox")
        self.bias_voltage_spinbox.setDecimals(3)
        self.bias_voltage_spinbox.setMinimum(-32.768000000000001)
        self.bias_voltage_spinbox.setMaximum(32.767000000000003)
        self.bias_voltage_spinbox.setSingleStep(0.001000000000000)

        self.gridLayout.addWidget(self.bias_voltage_spinbox, 6, 1, 1, 1)

        self.label_13 = QLabel(Param_CT)
        self.label_13.setObjectName(u"label_13")
        sizePolicy1 = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.label_13.sizePolicy().hasHeightForWidth())
        self.label_13.setSizePolicy(sizePolicy1)
        self.label_13.setFont(font)
        self.label_13.setAlignment(Qt.AlignLeading|Qt.AlignLeft|Qt.AlignTop)

        self.gridLayout.addWidget(self.label_13, 7, 0, 1, 1)

        self.params = QGridLayout()
        self.params.setObjectName(u"params")
        self.calp3 = QDoubleSpinBox(Param_CT)
        self.calp3.setObjectName(u"calp3")
        self.calp3.setMinimum(-2.560000000000000)
        self.calp3.setMaximum(2.540000000000000)
        self.calp3.setSingleStep(0.020000000000000)

        self.params.addWidget(self.calp3, 3, 3, 1, 1)

        self.label_14 = QLabel(Param_CT)
        self.label_14.setObjectName(u"label_14")
        self.label_14.setFont(font)
        self.label_14.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.params.addWidget(self.label_14, 1, 1, 1, 1)

        self.calv3 = QDoubleSpinBox(Param_CT)
        self.calv3.setObjectName(u"calv3")
        self.calv3.setMinimum(-2.560000000000000)
        self.calv3.setMaximum(2.540000000000000)
        self.calv3.setSingleStep(0.020000000000000)

        self.params.addWidget(self.calv3, 3, 2, 1, 1)

        self.label_17 = QLabel(Param_CT)
        self.label_17.setObjectName(u"label_17")
        self.label_17.setFont(font)
        self.label_17.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.params.addWidget(self.label_17, 4, 1, 1, 1)

        self.calv2 = QDoubleSpinBox(Param_CT)
        self.calv2.setObjectName(u"calv2")
        self.calv2.setMinimum(-2.560000000000000)
        self.calv2.setMaximum(2.540000000000000)
        self.calv2.setSingleStep(0.020000000000000)

        self.params.addWidget(self.calv2, 2, 2, 1, 1)

        self.calv1 = QDoubleSpinBox(Param_CT)
        self.calv1.setObjectName(u"calv1")
        self.calv1.setMinimum(-2.560000000000000)
        self.calv1.setMaximum(2.540000000000000)
        self.calv1.setSingleStep(0.020000000000000)

        self.params.addWidget(self.calv1, 1, 2, 1, 1)

        self.calp2 = QDoubleSpinBox(Param_CT)
        self.calp2.setObjectName(u"calp2")
        self.calp2.setMinimum(-2.560000000000000)
        self.calp2.setMaximum(2.540000000000000)
        self.calp2.setSingleStep(0.020000000000000)

        self.params.addWidget(self.calp2, 2, 3, 1, 1)

        self.label_16 = QLabel(Param_CT)
        self.label_16.setObjectName(u"label_16")
        self.label_16.setFont(font)
        self.label_16.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.params.addWidget(self.label_16, 3, 1, 1, 1)

        self.label_15 = QLabel(Param_CT)
        self.label_15.setObjectName(u"label_15")
        self.label_15.setFont(font)
        self.label_15.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.params.addWidget(self.label_15, 2, 1, 1, 1)

        self.calp4 = QDoubleSpinBox(Param_CT)
        self.calp4.setObjectName(u"calp4")
        self.calp4.setMinimum(-2.560000000000000)
        self.calp4.setMaximum(2.540000000000000)
        self.calp4.setSingleStep(0.020000000000000)

        self.params.addWidget(self.calp4, 4, 3, 1, 1)

        self.label_20 = QLabel(Param_CT)
        self.label_20.setObjectName(u"label_20")
        self.label_20.setFont(font)
        self.label_20.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.params.addWidget(self.label_20, 0, 1, 1, 1)

        self.calp1 = QDoubleSpinBox(Param_CT)
        self.calp1.setObjectName(u"calp1")
        self.calp1.setMinimum(-2.560000000000000)
        self.calp1.setMaximum(2.540000000000000)
        self.calp1.setSingleStep(0.020000000000000)

        self.params.addWidget(self.calp1, 1, 3, 1, 1)

        self.calv4 = QDoubleSpinBox(Param_CT)
        self.calv4.setObjectName(u"calv4")
        self.calv4.setMinimum(-2.560000000000000)
        self.calv4.setMaximum(2.540000000000000)
        self.calv4.setSingleStep(0.020000000000000)

        self.params.addWidget(self.calv4, 4, 2, 1, 1)

        self.label_18 = QLabel(Param_CT)
        self.label_18.setObjectName(u"label_18")
        self.label_18.setFont(font)

        self.params.addWidget(self.label_18, 0, 2, 1, 1)

        self.label_19 = QLabel(Param_CT)
        self.label_19.setObjectName(u"label_19")
        self.label_19.setFont(font)

        self.params.addWidget(self.label_19, 0, 3, 1, 1)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)

        self.params.addItem(self.horizontalSpacer, 2, 0, 1, 1)


        self.gridLayout.addLayout(self.params, 8, 0, 1, 2)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.gridLayout.addItem(self.verticalSpacer, 9, 0, 1, 1)

        QWidget.setTabOrder(self.current_spinbox, self.size_spinbox)
        QWidget.setTabOrder(self.size_spinbox, self.output_voltage_spinbox)
        QWidget.setTabOrder(self.output_voltage_spinbox, self.phase_spinbox)
        QWidget.setTabOrder(self.phase_spinbox, self.voltage_temp_coeff_spinbox)
        QWidget.setTabOrder(self.voltage_temp_coeff_spinbox, self.phase_temp_coeff_spinbox)
        QWidget.setTabOrder(self.phase_temp_coeff_spinbox, self.calv1)
        QWidget.setTabOrder(self.calv1, self.calp1)
        QWidget.setTabOrder(self.calp1, self.calv2)
        QWidget.setTabOrder(self.calv2, self.calp2)
        QWidget.setTabOrder(self.calp2, self.calv3)
        QWidget.setTabOrder(self.calv3, self.calp3)
        QWidget.setTabOrder(self.calp3, self.calv4)
        QWidget.setTabOrder(self.calv4, self.calp4)

        self.retranslateUi(Param_CT)

        QMetaObject.connectSlotsByName(Param_CT)
    # setupUi

    def retranslateUi(self, Param_CT):
        self.label_9.setText(QCoreApplication.translate("Param_CT", u"Rated Current", None))
        self.current_spinbox.setSuffix(QCoreApplication.translate("Param_CT", u" A", None))
        self.label_5.setText(QCoreApplication.translate("Param_CT", u"Size", None))
        self.size_spinbox.setSuffix(QCoreApplication.translate("Param_CT", u" mm", None))
        self.label_10.setText(QCoreApplication.translate("Param_CT", u"Output Voltage at Rated Current", None))
        self.output_voltage_spinbox.setSuffix(QCoreApplication.translate("Param_CT", u" V", None))
        self.label_23.setText(QCoreApplication.translate("Param_CT", u"Output Phase at Rated Current", None))
        self.phase_spinbox.setSuffix(QCoreApplication.translate("Param_CT", u" \u00b0", None))
        self.label_11.setText(QCoreApplication.translate("Param_CT", u"Voltage Temperature Coefficient", None))
        self.voltage_temp_coeff_spinbox.setSuffix(QCoreApplication.translate("Param_CT", u" ppm/\u00b0C", None))
        self.label_12.setText(QCoreApplication.translate("Param_CT", u"Phase Temperature Coefficient", None))
        self.phase_temp_coeff_spinbox.setSuffix(QCoreApplication.translate("Param_CT", u" m\u00b0/\u00b0C", None))
        self.label_21.setText(QCoreApplication.translate("Param_CT", u"Bias voltage", None))
        self.bias_voltage_spinbox.setSuffix(QCoreApplication.translate("Param_CT", u" mV", None))
        self.label_13.setText(QCoreApplication.translate("Param_CT", u"Calibration Table", None))
        self.calp3.setSuffix(QCoreApplication.translate("Param_CT", u" \u00b0", None))
        self.label_14.setText(QCoreApplication.translate("Param_CT", u"1.5%", None))
        self.calv3.setSuffix(QCoreApplication.translate("Param_CT", u" %", None))
        self.label_17.setText(QCoreApplication.translate("Param_CT", u"50%", None))
        self.calv2.setSuffix(QCoreApplication.translate("Param_CT", u" %", None))
        self.calv1.setSuffix(QCoreApplication.translate("Param_CT", u" %", None))
        self.calp2.setSuffix(QCoreApplication.translate("Param_CT", u" \u00b0", None))
        self.label_16.setText(QCoreApplication.translate("Param_CT", u"15%", None))
        self.label_15.setText(QCoreApplication.translate("Param_CT", u"5%", None))
        self.calp4.setSuffix(QCoreApplication.translate("Param_CT", u" \u00b0", None))
        self.label_20.setText(QCoreApplication.translate("Param_CT", u"Level", None))
        self.calp1.setSuffix(QCoreApplication.translate("Param_CT", u" \u00b0", None))
        self.calv4.setSuffix(QCoreApplication.translate("Param_CT", u" %", None))
        self.label_18.setText(QCoreApplication.translate("Param_CT", u"Voltage Adjustment", None))
        self.label_19.setText(QCoreApplication.translate("Param_CT", u"Phase Adjustment", None))
        pass
    # retranslateUi




#
# Copyright (c) 2020-2021 eGauge Systems LLC
#       1644 Conestoga St, Suite 2
#       Boulder, CO 80301
#       voice: 720-545-9767
#       email: davidm@egauge.net
#
# All rights reserved.
#
# MIT License
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
from PySide2.QtWidgets import QMessageBox, QWidget

import egauge.ctid as ctid

def invalid_double(string):
    try:
        float(string)
    except ValueError:
        return True
    return False

class Sensor_Params:
    def __init__(self, ui, parent, title='Sensor Parameters'):
        self.parent = parent
        self.ui = ui
        self.widget = QWidget(parent)
        self.widget.setVisible(False)
        self.ui.setupUi(self.widget)
        self.title = title

    def activate(self):
        '''Display our set of sensor parameters.'''
        self.widget.setVisible(True)
        self.parent.setTitle(self.title)

    def deactivate(self):
        '''Hide our set of sensor parameters.'''
        self.widget.setVisible(False)
        self.parent.setTitle('Sensor Parameters')

    def set_input_enabled(self, enabled):
        '''Enable or disable inputs.'''

    def load(self, template):
        '''Load sensor parameters from a template.'''

    def save(self, template):
        '''Save sensor parameters in the template dictionary, without
        validation.

        '''

    def validate(self, template):
        '''Validate the parameter values in the template.  If there are any
        errors, focus on the input element with the erroneous value
        and return False, otherwise return True.

        '''
        return True

    def encoder_argv(self, template):
        '''Convert the parameter values to the argument list required for the
        CTid-encoder program.'''
        return []

    def table_to_template(self, table, template):
        '''Extract sensor parameters from table and add them to the template.'''

class CT(Sensor_Params):
    def set_input_enabled(self, enabled):
        self.ui.current_spinbox.setEnabled(enabled)
        self.ui.size_spinbox.setEnabled(enabled)
        self.ui.output_voltage_spinbox.setEnabled(enabled)
        self.ui.bias_voltage_spinbox.setEnabled(enabled)
        self.ui.phase_spinbox.setEnabled(enabled)
        self.ui.voltage_temp_coeff_spinbox.setEnabled(enabled)
        self.ui.phase_temp_coeff_spinbox.setEnabled(enabled)
        for row in range(4):
            v_cal = self.ui.__dict__['calv%d' % (row + 1)]
            p_cal = self.ui.__dict__['calp%d' % (row + 1)]
            v_cal.setEnabled(enabled)
            p_cal.setEnabled(enabled)

    def load(self, template):
        self.ui.current_spinbox.setValue(template['current'])
        self.ui.size_spinbox.setValue(template['size'])
        self.ui.output_voltage_spinbox.setValue(template['v_output'])
        self.ui.bias_voltage_spinbox.setValue(template['v_bias'])
        self.ui.phase_spinbox.setValue(template['phi'])
        self.ui.voltage_temp_coeff_spinbox.setValue(template['v_temp_coeff'])
        self.ui.phase_temp_coeff_spinbox.setValue(template['phi_temp_coeff'])

        cal_table = template['cal_table']
        for row in range(4):
            levels = ['1.5', '5', '15', '50']
            v_cal = self.ui.__dict__['calv%d' % (row + 1)]
            p_cal = self.ui.__dict__['calp%d' % (row + 1)]
            if levels[row] not in cal_table:
                v_cal.setValue(0.0)
                p_cal.setValue(0.0)
                continue
            adj = cal_table[levels[row]]
            v_cal.setValue(adj[0])
            p_cal.setValue(adj[1])

    def save(self, template):
        template['current'] = self.ui.current_spinbox.value()
        template['size'] = self.ui.size_spinbox.value()
        template['v_output'] = self.ui.output_voltage_spinbox.value()
        template['v_bias'] = self.ui.bias_voltage_spinbox.value()
        template['phi'] = self.ui.phase_spinbox.value()
        template['v_temp_coeff'] = self.ui.voltage_temp_coeff_spinbox.value()
        template['phi_temp_coeff'] = self.ui.phase_temp_coeff_spinbox.value()
        cal_table = {}
        for row in range(4):
            levels = ['1.5', '5', '15', '50']
            v_cal = self.ui.__dict__['calv%d' % (row + 1)].value()
            p_cal = self.ui.__dict__['calp%d' % (row + 1)].value()
            if v_cal == 0.0 and p_cal == 0.0:
                continue
            cal_table[levels[row]] = (v_cal, p_cal)
        template['cal_table'] = cal_table

    def encoder_argv(self, template):
        argv = ['-s', '%.1f' % template['size'],
                '-c', '%.1f' % template['current'],
                '-v', '%.5f' % template['v_output'],
                '-b', '%.6f' % (1e-3 * template['v_bias']),
                '-p', '%.2f' % template['phi'],
                '-t', '%d' % template['v_temp_coeff'],
                '-T', '%.1f' % template['phi_temp_coeff']]
        for lvl, adj in template['cal_table'].items():
            argv += ['-a', '%s:%.3f/%.3f' % (lvl, adj[0], adj[1])]
        return argv

    def table_to_template(self, table, template):
        template['current'] = table.rated_current
        template['size'] = table.size
        template['v_output'] = table.voltage_at_rated_current
        template['v_bias'] = 1e3 * table.bias_voltage
        template['phi'] = table.phase_at_rated_current
        template['v_temp_coeff'] = table.voltage_temp_coeff
        template['phi_temp_coeff'] = table.phase_temp_coeff
        cal_table = {}
        for row in range(4):
            levels = [1.5, 5, 15, 50]
            v_cal = table.cal_table[levels[row]][0]
            p_cal = table.cal_table[levels[row]][1]
            if v_cal == 0.0 and p_cal == 0.0:
                continue
            cal_table[str(levels[row])] = (v_cal, p_cal)
        template['cal_table'] = cal_table

class Linear(Sensor_Params):
    def __init__(self, ui, parent, title):
        super().__init__(ui, parent, title)
        max_float = 3.4028235e38
        self.ui.scale_spinbox.setRange(-max_float, max_float)
        self.ui.offset_spinbox.setRange(-max_float, max_float)
        for _, unit_desc in ctid.SENSOR_UNITS:
            self.ui.unit_comboBox.addItem(unit_desc)
        self.ui.unit_comboBox.currentIndexChanged.connect(self._unit_changed)

    def _unit_changed(self):
        unit_code = self.ui.unit_comboBox.currentIndex()
        unit = ctid.get_sensor_unit(unit_code)
        if unit == '':
            scale_unit = ' 1/V'
            offset_unit = ''
        else:
            scale_unit = ' %s/V' % unit
            offset_unit = ' %s' % unit
        self.ui.scale_spinbox.setSuffix(scale_unit)
        self.ui.offset_spinbox.setSuffix(offset_unit)

    def set_input_enabled(self, enabled):
        self.ui.unit_comboBox.setEnabled(enabled)
        self.ui.scale_spinbox.setEnabled(enabled)
        self.ui.offset_spinbox.setEnabled(enabled)
        self.ui.delay_spinbox.setEnabled(enabled)

    def load(self, template):
        self.ui.unit_comboBox.setCurrentIndex(template['unit'])
        self.ui.scale_spinbox.setValue(template['scale'])
        self.ui.offset_spinbox.setValue(template['offset'])
        self.ui.delay_spinbox.setValue(template['delay'])

    def save(self, template):
        template['unit'] = self.ui.unit_comboBox.currentIndex()
        template['scale'] = self.ui.scale_spinbox.value()
        template['offset'] = self.ui.offset_spinbox.value()
        template['delay'] = self.ui.delay_spinbox.value()

    def encoder_argv(self, template):
        return ['--unit', str(template['unit']),
                '--scale', str(template['scale']),
                '--offset', str(template['offset']),
                '--delay', str(template['delay'])]

    def table_to_template(self, table, template):
        template['unit'] = table.sensor_unit
        template['scale'] = table.scale
        template['offset'] = table.offset
        template['delay'] = table.delay

class Temp(Sensor_Params):
    def set_input_enabled(self, enabled):
        self.ui.scale_spinbox.setEnabled(enabled)
        self.ui.offset_spinbox.setEnabled(enabled)

    def load(self, template):
        self.ui.scale_spinbox.setValue(template['scale'])
        self.ui.offset_spinbox.setValue(template['offset'])

    def save(self, template):
        template['scale'] = self.ui.scale_spinbox.value()
        template['offset'] = self.ui.offset_spinbox.value()

    def encoder_argv(self, template):
        # convert from °C/V to °C/V:
        return ['--scale', str(template['scale']),
                '--offset', str(template['offset'])]

    def table_to_template(self, table, template):
        template['scale'] = table.scale
        template['offset'] = table.offset

class NTC(Sensor_Params):
    '''Note that the NTC parameters are line editors rather than spinboxes
    because their value can be any double value, including some really
    tiny values that are best entered via exponent notation.  The QT4
    double spinbox is not well setup for such values.

    '''

    def set_input_enabled(self, enabled):
        self.ui.ntc_a_lineEdit.setEnabled(enabled)
        self.ui.ntc_b_lineEdit.setEnabled(enabled)
        self.ui.ntc_c_lineEdit.setEnabled(enabled)
        self.ui.ntc_m_lineEdit.setEnabled(enabled)
        self.ui.ntc_n_lineEdit.setEnabled(enabled)
        self.ui.ntc_k_lineEdit.setEnabled(enabled)

    def load(self, template):
        self.ui.ntc_a_lineEdit.setText(template['ntc_a'])
        self.ui.ntc_b_lineEdit.setText(template['ntc_b'])
        self.ui.ntc_c_lineEdit.setText(template['ntc_c'])
        self.ui.ntc_m_lineEdit.setText(template['ntc_m'])
        self.ui.ntc_n_lineEdit.setText(template['ntc_n'])
        self.ui.ntc_k_lineEdit.setText(template['ntc_k'])

    def save(self, template):
        template['ntc_a'] = self.ui.ntc_a_lineEdit.text()
        template['ntc_b'] = self.ui.ntc_b_lineEdit.text()
        template['ntc_c'] = self.ui.ntc_c_lineEdit.text()
        template['ntc_m'] = self.ui.ntc_m_lineEdit.text()
        template['ntc_n'] = self.ui.ntc_n_lineEdit.text()
        template['ntc_k'] = self.ui.ntc_k_lineEdit.text()

    def validate(self, template):
        bad_field = None
        if invalid_double(template['ntc_a']):
            bad_field = self.ui.ntc_a_lineEdit
        elif invalid_double(template['ntc_b']):
            bad_field = self.ui.ntc_b_lineEdit
        elif invalid_double(template['ntc_c']):
            bad_field = self.ui.ntc_d_lineEdit
        elif invalid_double(template['ntc_m']):
            bad_field = self.ui.ntc_m_lineEdit
        elif invalid_double(template['ntc_n']):
            bad_field = self.ui.ntc_r1_lineEdit
        elif invalid_double(template['ntc_k']):
            bad_field = self.ui.ntc_r1_lineEdit

        if bad_field is not None:
            QMessageBox.warning(self, 'Invalid number',
                                'Please enter a value number.',
                                QMessageBox.Ok)
            bad_field.setFocus()
            return False

        return True

    def encoder_argv(self, template):
        return ['--ntc-a', str(template['ntc_a']),
                '--ntc-b', str(template['ntc_b']),
                '--ntc-c', str(template['ntc_c']),
                '--ntc-m', str(template['ntc_m']),
                '--ntc-n', str(template['ntc_n']),
                '--ntc-k', str(template['ntc_k'])]

    def table_to_template(self, table, template):
        template['ntc_a'] = str(table.ntc_a)
        template['ntc_b'] = str(table.ntc_b)
        template['ntc_c'] = str(table.ntc_c)
        template['ntc_m'] = str(table.ntc_m)
        template['ntc_n'] = str(table.ntc_n)
        template['ntc_k'] = str(table.ntc_k)

class Pulse(Sensor_Params):
    '''Note that the NTC parameters are line editors rather than spinboxes
    because their value can be any double value, including some really
    tiny values that are best entered via exponent notation.  The QT4
    double spinbox is not well setup for such values.

    '''

    def set_input_enabled(self, enabled):
        self.ui.threshold_spinbox.setEnabled(enabled)
        self.ui.hysteresis_spinbox.setEnabled(enabled)
        self.ui.debounce_spinbox.setEnabled(enabled)
        self.ui.edge_comboBox.setEnabled(enabled)

    def load(self, template):
        self.ui.threshold_spinbox.setValue(template['threshold'])
        self.ui.hysteresis_spinbox.setValue(template['hysteresis'])
        self.ui.debounce_spinbox.setValue(template['debounce_time'])
        idx = 0		# default to 'rising edge'
        edge = template['edge_mask']
        if edge == 'falling':
            idx = 1
        elif edge == 'both':
            idx = 2
        self.ui.edge_comboBox.setCurrentIndex(idx)

    def save(self, template):
        mask = ['rising', 'falling', 'both']
        template['threshold'] = self.ui.threshold_spinbox.value()
        template['hysteresis'] = self.ui.hysteresis_spinbox.value()
        template['debounce_time'] = self.ui.debounce_spinbox.value()
        template['edge_mask'] = mask[self.ui.edge_comboBox.currentIndex()]

    def encoder_argv(self, template):
        return ['--threshold', str(1e-3 * template['threshold']),
                '--hysteresis', str(1e-3 * template['hysteresis']),
                '--debounce-time', str(template['debounce_time']),
                '--edge-mask', template['edge_mask']]

    def table_to_template(self, table, template):
        mask = ['none', 'rising', 'falling', 'both']
        template['threshold'] = 1e3 * table.threshold
        template['hysteresis'] = 1e3 * table.hysteresis
        template['debounce_time'] = table.debounce_time
        template['edge_mask'] = mask[table.edge_mask]



#
# Copyright (c) 2020-2021 eGauge Systems LLC
#       1644 Conestoga St, Suite 2
#       Boulder, CO 80301
#       voice: 720-545-9767
#       email: davidm@egauge.net
#
# All rights reserved.
#
# MIT License
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# pylint: disable=no-name-in-module
#
'''Serial number allocation via the eGauge serial-number API.'''

import logging
import os

from PySide2.QtWidgets import QMessageBox

from egauge import ctid
from egauge import webapi

from egauge.webapi.cloud.credentials import Credentials_Manager, LoginCanceled

from ctid_programmer import sn

log = logging.getLogger(__name__)

class Manager(sn.Manager):
    def auth_wrapper(self, method, *args, **kwargs):
        self.credentials_manager.previous_login_failed = False
        while True:
            try:
                return method(*args, *kwargs)
            except webapi.json_api.UnauthenticatedError:
                self.credentials_manager.previous_login_failed = True
            except LoginCanceled:
                return None

    def __init__(self, ui, state_directory_path):
        SerialNumberWithAuth \
            = webapi.decorate_public(webapi.cloud.SerialNumber,
                                     self.auth_wrapper)
        self.credentials_manager = Credentials_Manager(ui)
        self.sn_api = SerialNumberWithAuth(
            auth=webapi.auth.TokenAuth(ask=self.credentials_manager.ask))
        super().__init__(ui, os.path.join(state_directory_path,
                                          'egauge_sn_api.bin'))

    def has_calibration_data(self):
        '''Should return True if this serial-number manager can retrieve
        calibration data with get_calibration_data(), False otherwise.

        '''
        return True

    def set_product(self, manufacturer, model):
        log.debug('set_product: manufacturer=%s, model=%s',
                  manufacturer, model)

        mfg = ctid.mfg_short_name(manufacturer)
        if not super().set_product(mfg, model):
            return False

        model_list = None
        try:
            model_list = self.sn_api.get_models()
        except webapi.Error:
            pass

        if model_list is None:
            QMessageBox.critical(self.ui, 'Serial-number service failed',
                                 'Serial-number service failed to return '
                                 'model-name list.  '
                                 'Reverting to manual '
                                 'serial numbers.', QMessageBox.Ok)
            return False

        found = False
        for r in model_list:
            if r['name'] == self.product:
                found = True
                break
        if found:
            return True

        choice = QMessageBox \
                 .question(self.ui,
                           'Create Serial-Number Record?',
                           ('The eGauge Serial-Number service has no record '
                            'of product \"%s %s\".  Would you like to '
                            'create one?'
                            % (mfg, model)), QMessageBox.Yes | QMessageBox.No)
        if choice == QMessageBox.No:
            return False

        result = None
        try:
            result = self.sn_api.create_model(self.product, 0xffffff)
        except webapi.Error:
            log.exception('sn_api.create_model() for model "%s" failed',
                          self.product)
        return result

    def set_preferences(self, prefs):
        pass

    def get(self):
        # see if we already have a serial-number for this product:
        sn = super().get()
        if sn is not None:
            log.debug('get: product=%s; using existing SN %s',
                      self.product, sn)
            return sn

        log.debug('get: product=%s; allocating new SN ', self.product)
        return self._allocate_sn()

    def commit(self, sn, info, auto_serial_number):
        meta = {}
        try:
            ret = self.sn_api.get_metadata(self.product, sn)
            if ret is not None:
                meta = ret
        except webapi.Error:
            log.exception('commit: no metadata found for %s SN %s.',
                          self.product, sn)
        meta['ctid'] = info
        try:
            self.sn_api.set_metadata(self.product, sn, meta)
        except webapi.Error as e:
            QMessageBox.warning(self.ui, 'Failed to save CTid info',
                                'Failed to save meta data for product %s '
                                'serial-number %s: %s.' \
                                % (self.product, sn, e), QMessageBox.Ok)

        if auto_serial_number:
            self._allocate_sn()		# allocate serial number to use next
        else:
            super().set(None)

    def get_cal_data(self, sn):
        try:
            meta = self.sn_api.get_metadata(self.product, sn)
        except webapi.Error:
            log.exception('Failed to get cal data for %s SN %s.',
                          self.product, sn)
            return None

        if meta is None:
            return None
        if 'cal' in meta:
            cal = meta['cal']
        elif 'ntc' in meta:
            # for backwards compatibility:
            cal = {}
            for key, value in meta['ntc'].items():
                cal['ntc_%s' % key] = str(value)
        else:
            return None
        return cal

    def _allocate_sn(self):
        # allocate a new serial number:
        super().set(None)
        try:
            sn = self.sn_api.allocate(self.product)
            if sn is None:
                return None
        except webapi.Error as e:
            log.exception('Failed to allocate SN for product %s.',
                          self.product)
            if len(e.args) > 2 and isinstance(e.args[2], list) \
               and len(e.args[2]) >= 1:
                errors = e.args[2]
                # If this product is out of serial-numbers, let the user know:
                if errors[0] == 'Maximum serial number reached':
                    raise sn.SpaceExhausted
                QMessageBox.critical(self.ui, 'Serial-number Failure',
                                     'Failed to allocate serial-number: %s'
                                     % '\n'.join(errors),
                                     QMessageBox.Ok)
            else:
                QMessageBox.critical(self.ui, 'Serial-number Failure',
                                     'Failed to allocate serial-number: %s'
                                     % e, QMessageBox.Ok)
            return None

        super().set(sn)
        return sn



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'param_ntc.ui'
##
## Created by: Qt User Interface Compiler version 5.15.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_Param_NTC(object):
    def setupUi(self, Param_NTC):
        if not Param_NTC.objectName():
            Param_NTC.setObjectName(u"Param_NTC")
        Param_NTC.resize(280, 292)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Param_NTC.sizePolicy().hasHeightForWidth())
        Param_NTC.setSizePolicy(sizePolicy)
        self.verticalLayout = QVBoxLayout(Param_NTC)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setContentsMargins(-1, 24, -1, -1)
        self.groupBox = QGroupBox(Param_NTC)
        self.groupBox.setObjectName(u"groupBox")
        self.groupBox.setFlat(False)
        self.gridLayout = QGridLayout(self.groupBox)
        self.gridLayout.setObjectName(u"gridLayout")
        self.label_9 = QLabel(self.groupBox)
        self.label_9.setObjectName(u"label_9")
        font = QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_9.setFont(font)

        self.gridLayout.addWidget(self.label_9, 0, 0, 1, 1)

        self.label_10 = QLabel(self.groupBox)
        self.label_10.setObjectName(u"label_10")
        self.label_10.setFont(font)

        self.gridLayout.addWidget(self.label_10, 2, 0, 1, 1)

        self.label_5 = QLabel(self.groupBox)
        self.label_5.setObjectName(u"label_5")
        self.label_5.setFont(font)

        self.gridLayout.addWidget(self.label_5, 1, 0, 1, 1)

        self.label_2 = QLabel(self.groupBox)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setOpenExternalLinks(True)

        self.gridLayout.addWidget(self.label_2, 3, 0, 1, 1)

        self.ntc_b_lineEdit = QLineEdit(self.groupBox)
        self.ntc_b_lineEdit.setObjectName(u"ntc_b_lineEdit")

        self.gridLayout.addWidget(self.ntc_b_lineEdit, 1, 1, 1, 1)

        self.ntc_a_lineEdit = QLineEdit(self.groupBox)
        self.ntc_a_lineEdit.setObjectName(u"ntc_a_lineEdit")

        self.gridLayout.addWidget(self.ntc_a_lineEdit, 0, 1, 1, 1)

        self.ntc_c_lineEdit = QLineEdit(self.groupBox)
        self.ntc_c_lineEdit.setObjectName(u"ntc_c_lineEdit")

        self.gridLayout.addWidget(self.ntc_c_lineEdit, 2, 1, 1, 1)


        self.verticalLayout.addWidget(self.groupBox)

        self.gridLayout_2 = QGridLayout()
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.label_4 = QLabel(Param_NTC)
        self.label_4.setObjectName(u"label_4")
        self.label_4.setFont(font)
        self.label_4.setTextFormat(Qt.AutoText)

        self.gridLayout_2.addWidget(self.label_4, 0, 0, 1, 1)

        self.label = QLabel(Param_NTC)
        self.label.setObjectName(u"label")
        self.label.setFont(font)
        self.label.setTextFormat(Qt.AutoText)

        self.gridLayout_2.addWidget(self.label, 1, 0, 1, 1)

        self.label_3 = QLabel(Param_NTC)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setFont(font)
        self.label_3.setTextFormat(Qt.AutoText)

        self.gridLayout_2.addWidget(self.label_3, 2, 0, 1, 1)

        self.ntc_m_lineEdit = QLineEdit(Param_NTC)
        self.ntc_m_lineEdit.setObjectName(u"ntc_m_lineEdit")
        sizePolicy1 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.ntc_m_lineEdit.sizePolicy().hasHeightForWidth())
        self.ntc_m_lineEdit.setSizePolicy(sizePolicy1)

        self.gridLayout_2.addWidget(self.ntc_m_lineEdit, 0, 1, 1, 1)

        self.ntc_n_lineEdit = QLineEdit(Param_NTC)
        self.ntc_n_lineEdit.setObjectName(u"ntc_n_lineEdit")

        self.gridLayout_2.addWidget(self.ntc_n_lineEdit, 1, 1, 1, 1)

        self.ntc_k_lineEdit = QLineEdit(Param_NTC)
        self.ntc_k_lineEdit.setObjectName(u"ntc_k_lineEdit")

        self.gridLayout_2.addWidget(self.ntc_k_lineEdit, 2, 1, 1, 1)


        self.verticalLayout.addLayout(self.gridLayout_2)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.verticalLayout.addItem(self.verticalSpacer)

        QWidget.setTabOrder(self.ntc_a_lineEdit, self.ntc_b_lineEdit)
        QWidget.setTabOrder(self.ntc_b_lineEdit, self.ntc_c_lineEdit)
        QWidget.setTabOrder(self.ntc_c_lineEdit, self.ntc_m_lineEdit)
        QWidget.setTabOrder(self.ntc_m_lineEdit, self.ntc_n_lineEdit)
        QWidget.setTabOrder(self.ntc_n_lineEdit, self.ntc_k_lineEdit)

        self.retranslateUi(Param_NTC)

        QMetaObject.connectSlotsByName(Param_NTC)
    # setupUi

    def retranslateUi(self, Param_NTC):
        self.groupBox.setTitle(QCoreApplication.translate("Param_NTC", u"Steinhart-Hart Coefficients", None))
        self.label_9.setText(QCoreApplication.translate("Param_NTC", u"A", None))
        self.label_10.setText(QCoreApplication.translate("Param_NTC", u"C", None))
        self.label_5.setText(QCoreApplication.translate("Param_NTC", u"B", None))
        self.label_2.setText(QCoreApplication.translate("Param_NTC", u"<a href=\"https://www.thinksrs.com/downloads/programs/therm%20calc/ntccalibrator/ntccalculator.html\">Look up coefficients</a>", None))
        self.ntc_b_lineEdit.setPlaceholderText(QCoreApplication.translate("Param_NTC", u"[1/K]", None))
        self.ntc_a_lineEdit.setPlaceholderText(QCoreApplication.translate("Param_NTC", u"[1/K]", None))
        self.ntc_c_lineEdit.setPlaceholderText(QCoreApplication.translate("Param_NTC", u"[1/K]", None))
        self.label_4.setText(QCoreApplication.translate("Param_NTC", u"M", None))
        self.label.setText(QCoreApplication.translate("Param_NTC", u"N", None))
        self.label_3.setText(QCoreApplication.translate("Param_NTC", u"K", None))
        self.ntc_m_lineEdit.setPlaceholderText(QCoreApplication.translate("Param_NTC", u"M parameter [V]", None))
        self.ntc_n_lineEdit.setPlaceholderText(QCoreApplication.translate("Param_NTC", u"N parameter [V/\u03a9]", None))
        self.ntc_k_lineEdit.setPlaceholderText(QCoreApplication.translate("Param_NTC", u"K parameter [1/\u03a9]", None))
        pass
    # retranslateUi




# Resource object code (Python 3)
# Created by: object code
# Created by: The Resource Compiler for Qt version 5.15.1
# WARNING! All changes made in this file will be lost!

from PySide2 import QtCore

qt_resource_data = b"\
\x00\x00\x05l\
\x00\
\x00\x17Ex\x9c\xedX\xdbn\xdb6\x18\xbe\xcfSh\
\xeaE\x1b,\xa2H\x8a\x92%\xc5v\x81\xb5(P`\
\xdb\xc5\x96a\x97\x03#\xd16QI\x14(:v\xfa\
P{\x88=\xd9~\xead+Q\x80bE\xb7v-\
\x91\xc0\xe2\x7f\xe0\x7f\xfa\xf8\xc9\xc9\xf2\xe5\xb1,\x9c;\
\xa1\x1b\xa9\xaa\x95K\x10v\x1dQe*\x97\xd5v\xe5\
\xfev\xf3\xc6\x8b]\xa71\xbc\xcay\xa1*\xb1r+\
\xe5\xbe\x5c_,\xbf\xf3<\xe7\x95\x16\xdc\x88\xdc9H\
\xb3s\xdeV\xef\x9a\x8c\xd7\xc2y\xb13\xa6N}\xff\
p8 \xd9\x0b\x91\xd2[\xff\xd2\xf1\xbc\xf5\xc5\xc5\xb2\
\xb9\xdb^8\x8e\x03q\xab&\xcd\xb3\x95\xdb;\xd4{\
]\xb4\x86y\xe6\x8bB\x94\xa22\x8dO\x10\xf1\xdd\x93\
yv2\xcflty'2U\x96\xaajZ\xcf\xaa\
yvf\xac\xf3\xcdhm\xb39\x04\xad\x11I\x92\xc4\
\xc7\xd4\xa7\xd4\x03\x0b\xaf\xb9\xaf\x0c?zSW\xc8q\
\xce\x95b\x8c}\xd0\x9d,?\xcc*m\xa0\xa15\xfc\
\x8e\xe6\x83\x005j\xaf3\xb1\x01?\x81*a\xfc\xd7\
7\xafG\xa5\x87Qn\xf2\xb3c\x86~N\xa2N\x9a\
\x5c\xf1R45\xcfD\xe3\x0f\xf2\xd6\xff s\xb3[\
\xb94,\xcbv\xbf\x13r\xbb3g\x82;)\x0e?\
\xa8\xe3\xca\xc5\x0evh\x08?\x9d\xf8\x84\x0c\xd2\x0ad\
\xber\xa1\xb6\x98-\xbam\x1f$\x1d\x0d1J(\x0a\
\x9c\x17\x94\xe10d\xd1\x95C1\x89=\x1cx\x84\x5c\
\xb6.Cui\xae2\x9b\xee\xca\xcd\x8c\xcc\xbdBm\
\x15\x1a\xba6\x1e+\x8e\xb5\xd2\xc6\xdb\xc8Bt\xb6\xfe\
N\x95\xc2\xcf\xf9\x9d\xccK_l\xf9~+\xfc\xd6\xbf\
\xd6j\xabyY\x0a\xdd\xee\xff8\xdbk\xd1u\x19z\
\x92\x01T\xfcS\xbc\xba\x9a\x8fw\xcck\x98\x15\x8b\xd1\
b1\xab\xbf?\xd3\xaf\xc1`\x99\x8bMc\x0d\xbb\x06\
\xd9]\xcc\x88\xeb\xf8\xadr\xac\xd8\x96\x90\xdbN\x9fL\
oy\xd3M\xc8qj\xbe\x054\x17J\xaf\xdcg\x9b\
v\xf5\x8a[\xa5s\xa1\x07U\xd4\xae\x89J\xc1\xc4\xa5\
\xb9\xef\xeeo\x7f\xf6\x90\xb1=u\xd4\xe3y}\xb3\xe3\
\xb9:\x00\x1a\x1e*\xdf+U\x82\x18\xc5\x0f\x15\x19 \
\xc5\xa3\x0ca\xba =\x14\xce\xb5\x10\x8aE(\x0e\x93\
\x05~\xe4\x0ac\xdf\xdb\xcb\xed\xed+i\xe0\x02\xf5\x00\
<w\xdfkm\x0d\x0a~/\xa0\xe2\xf6\x83\xf4F\xcd\
N\x1d\xb6\xdavn\xc3\x8b\xb1u\xa3\xebAVP\x89\
\xd7\xc3\x9d\xc4\xf4QI\xbd\xc5p\x01\x08f\xc1\x13&\
P\x22\xc5\x8fj\xeb\x95\xb6\xd9\xc9S\x9e%?\xcaR\
\xbe\x17\x90\xe5\xd0\xef\x8d4 \xd6[YyF\xd5\xb3\
\xf2Bl\xcc\xacBw\xa9\xcehn\x951vBx\
\x80Z)\x0c\xcf\xb9\xe1'\x80\x0d\x92\x98\xb1\x16\xaa`\
\x04\xb4\x97\xfe\xf2\xfaM\xb7\x83}\x96\xa5\xbf+\xfd\xae\
\xdf\xc2\xb2\x06\xfcV\xed!\xa8\xbb\x1e\xc5\xcb<K\x81\
\xa8Jn\xd6\xb2\x04\xd4X\x8e\xfb\x1e\x88i\xe9\x9f\x14\
\x13cs_\x8b\xd3\xa1\xdd\xb1\xc3]\x9c\xa5\xfd<+\
\xa5u\xf2\x7f5\xb2(\xde\xda }ag\x87JS\
\x88u\x1b\xb3{\x1c\xaa\xf0\xfb2\xfa\x22\xfd\xb3*\x97\
\xfe\xd0\x85v\xb7}0\xb5\x82\xdf\x8ab\xe5\xfehq\
\xe6\x90\x873\xddj\xb5\xafK\x95\x8b\x1e\x89\xee\xa9\xb7\
\x13d\x1a\xcd\xab\xc6\xb6a\xe5\xb6\x8f\x05\xbc\x1c_x\
a\x84p\xb8\xc0\x84]ya\x80b\x12\x05\xf1\xe2r\
\x18\x84\x11G3\x14\x07\x8dL[\xe6^\xb954I\
\xe8\xbb\x01\xdc\x00zs_\x80|\xa3\xe0R\xb4\xcfi\
e\xdb]\x5c\xb7\x92;\xae%\xaf\xccDvh\xe1=\
\x115F\x0b\x93\xed\xa62\x00iJ)\x0a\x17v\xc5\
Q}\xbc.d%\xfa\xdb\x91\x12D\xc3\xcep\xc3K\
Y\xdc\xa7o\xf6f\xaf\xf9\xcdO\x22\xbf\xf6\x86\x06y\
\xddI\xb5\xc8\xe4Ff\xf0NVU\xfa\xfcdx\xe5\
\xfc\xdc\x06|>I\xd6+\xe4\x96\x83\x89h\xe6J\xf1\
\xe0\xdcyE\x05\xd4\xa1e6\xe8l\x03=\x0egU\
i\x06\x8c!\xf4u!\x0c|x\xb6\x93\xf0\x15&\xc5\
P\xd1\x018r*\xd0\xd2\xc0\xb3g\x87\x9a\x16\xda3\
\xb7\xfdAU\xb6S:-e\x9e\x17\xe2\x1a\xde9E\
\xfa\x0c\xb7\xab\xddx=\x8d\xa6\xe4\x1az\xa9\xde\xd9)\
T\xa2\x7f\xee('\xc5\x88F,\x8c\x83\x80\x8e\xc3\x03\
\x1a\x89\x02\x14\xd1$`d\x14\x02},\x00\x0e\x0c\xb3\
\xc5b\x14ZH\xd9DH\x90\xc0]]\x1a\xc8\xb9:\
\xdd\x9f\xf1-\xa2\x95\x05\x83\x9d\x93{\xd2\xb6\xbe\xd6\x01\
\x9c\xe9\x99|6\xfa\x13\xf1G\xa0=U\xd2\xfa\xd5\xd2\
o\x83\xc0\xfd\xb3\x89~V0&\x11J\x02X\x94\xb1\
/\x0d\xc6]n\xa2=\xc9k\x00\xc1\x80\xce\xe6\x0b\xc3\
\xf8\x82 \x16\x86\x04'\x13\x8cS\x940\x16\x122\x87\
\xf1\xf8c0\x1eM1\xfe8\xfa\x13\xf1?9\x0a\x19\
\x22v\xe1(\x81)\xfc\x8f@\xf7a\x80z\x929n\
>g\xe6\x98\xcc\xec\x1bs\xfc\xdb\xcc\x11\xa1\x00'Q\
D?\x909\x18\xa6\xff\x9c9\x18\xc6\x0f\x98\xe3Q\xf4\
\xff\x889\x12\x14\xb7_\xc2p\xf85\x12G\x87'\x92\
D\x94o\xe6\xf14C*2\xff\x9cY\x05\x10\x14\xd1\
\x98\xc6\xd1\xb7\xef#\x9f\x98U\x18\x0eq\x1c\x07\xc9\x84\
U\x18\x8a\x22L\x22v\xce*\x11C1]\x84\xc1\x1c\
\xa9\x9c\xbeQ\x9c\xfd%\x07\xa3(\xc4\x0b\x82p@\x17\
8\x88\xaf0\xcc\x14\xb0G\xe2\xf0\xf2c8\x88=\xe0\
\xa0G\xb9\xceg\xfb\xa9!\x1b!\x16\x13FX@\xbf\
F\x0az\x0aT\xeb\xbf\xfe|L3K\x7f\xbb\xbeX\
\xda\x7f\x81\xac/\xfe\x06r\x0f\x8e]\
"

qt_resource_name = b"\
\x00\x06\
\x07\x03}\xc3\
\x00i\
\x00m\x00a\x00g\x00e\x00s\
\x00\x0d\
\x00\xc0\xb3\x07\
\x00c\
\x00t\x00i\x00d\x00-\x00l\x00o\x00g\x00o\x00.\x00s\x00v\x00g\
"

qt_resource_struct = b"\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x01\x00\x00\x00\x01\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x01\x00\x00\x00\x02\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\x12\x00\x01\x00\x00\x00\x01\x00\x00\x00\x00\
\x00\x00\x01t\xfe\x85\x04\x1d\
"

def qInitResources():
    QtCore.qRegisterResourceData(0x03, qt_resource_struct, qt_resource_name, qt_resource_data)

def qCleanupResources():
    QtCore.qUnregisterResourceData(0x03, qt_resource_struct, qt_resource_name, qt_resource_data)

qInitResources()



#
# Copyright (c) 2020-2021 eGauge Systems LLC
#       1644 Conestoga St, Suite 2
#       Boulder, CO 80301
#       voice: 720-545-9767
#       email: davidm@egauge.net
#
# All rights reserved.
#
# MIT License
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
import pickle

from abc import ABC, abstractmethod	# ABC = abstract base class

class Error(Exception):
    pass

class SpaceExhausted(Error):
    pass

class Manager(ABC):

    def __init__(self, ui, state_path):
        '''Create a SN manager object.  UI is the QT user-interface object
        that can be used to interact with the user, e.g., to ask
        confirmation questions or to ask for authentication
        credentials, for example.  STATE_PATH is the path of a
        file that the manager can use to store state in.

        '''
        self.ui = ui
        self.path = state_path
        self.product = None
        self.serial = {}
        self._load_state()

    def has_calibration_data(self):
        '''Should return True if this serial-number manager can retrieve
        calibration data with get_calibration_data(), False otherwise.

        '''
        return False

    @abstractmethod
    def set_preferences(self, prefs):
        '''This method must be called whenever the SN manager's preferences
        may have changed.

        '''

    def set_product(self, manufacturer, model):
        '''Set the product for which to manage serial numbers.  Each product
        has its own serial-number space, so this function must be
        called at least once before any serial-numbers can be
        allocated.

        '''
        if manufacturer is None or model is None:
            self.product = None
            return True

        self.product = '%s-%s' % (manufacturer, model)
        return True

    def restore_product(self, product):
        '''Restore the product to PRODUCT.'''
        self.product = product

    def activate(self, sn):
        '''Activate automatic serial number generation using this SN manager.
        SN is the serial number that was most recently used in manual
        mode.  It may be None if the most recently used serial number
        is unavailable.  Some SN managers may choose to offer the next
        automatic serial number based on this number.

        '''

    def deactivate(self):
        '''Called when this SN manager is no longer used.  That is, when this
        method gets called when the user switches to manually entered
        serial-numbers.

        '''

    def get(self):
        '''Get the serial-number to be used next.'''
        if self.product is None:
            return None
        return self.serial.get(self.product)

    def set(self, sn):
        if self.product is None:
            return

        self.serial[self.product] = sn
        self._save_state()

    @abstractmethod
    def commit(self, sn, info, auto_serial_number):
        '''This is called after serial-number SN has been committed
        (programmed) into a device.  INFO must be a dictionary of
        information to be associated with this device.  Not all SN
        managers can preserve INFO.

        Note that this gets called even if the SN was manually
        selected (indicated by AUTH_SERIAL_NUMBER==False).  This is
        done such that if a device needs to be re-programmed (e.g.,
        with a different calibration), we get notified here with the
        latest INFO that is used for this SN.

        '''

    def get_cal_data(self, sn):
        '''Return the calibration data for the current product with serial
        number SN or None if no such data is available.

        '''
        return None

    def _load_state(self):
        try:
            with open(self.path, 'rb') as f:
                self.serial = pickle.load(f)
        except IOError:
            self.serial = {}

    def _save_state(self):
        with open(self.path, 'wb') as f:
            pickle.dump(self.serial, f)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'param_pulse.ui'
##
## Created by: Qt User Interface Compiler version 5.15.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_Param_Pulse(object):
    def setupUi(self, Param_Pulse):
        if not Param_Pulse.objectName():
            Param_Pulse.setObjectName(u"Param_Pulse")
        Param_Pulse.resize(343, 161)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Param_Pulse.sizePolicy().hasHeightForWidth())
        Param_Pulse.setSizePolicy(sizePolicy)
        self.verticalLayout = QVBoxLayout(Param_Pulse)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setContentsMargins(-1, 24, -1, -1)
        self.gridLayout = QGridLayout()
        self.gridLayout.setObjectName(u"gridLayout")
        self.threshold_spinbox = QDoubleSpinBox(Param_Pulse)
        self.threshold_spinbox.setObjectName(u"threshold_spinbox")
        self.threshold_spinbox.setDecimals(2)
        self.threshold_spinbox.setMaximum(655.350000000000023)

        self.gridLayout.addWidget(self.threshold_spinbox, 0, 1, 1, 1)

        self.label_9 = QLabel(Param_Pulse)
        self.label_9.setObjectName(u"label_9")
        font = QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_9.setFont(font)

        self.gridLayout.addWidget(self.label_9, 0, 0, 1, 1)

        self.label_5 = QLabel(Param_Pulse)
        self.label_5.setObjectName(u"label_5")
        self.label_5.setFont(font)

        self.gridLayout.addWidget(self.label_5, 1, 0, 1, 1)

        self.label_10 = QLabel(Param_Pulse)
        self.label_10.setObjectName(u"label_10")
        self.label_10.setFont(font)

        self.gridLayout.addWidget(self.label_10, 2, 0, 1, 1)

        self.label_23 = QLabel(Param_Pulse)
        self.label_23.setObjectName(u"label_23")
        self.label_23.setFont(font)

        self.gridLayout.addWidget(self.label_23, 3, 0, 1, 1)

        self.debounce_spinbox = QDoubleSpinBox(Param_Pulse)
        self.debounce_spinbox.setObjectName(u"debounce_spinbox")
        self.debounce_spinbox.setDecimals(0)
        self.debounce_spinbox.setMaximum(255.000000000000000)
        self.debounce_spinbox.setSingleStep(1.000000000000000)

        self.gridLayout.addWidget(self.debounce_spinbox, 2, 1, 1, 1)

        self.hysteresis_spinbox = QDoubleSpinBox(Param_Pulse)
        self.hysteresis_spinbox.setObjectName(u"hysteresis_spinbox")
        self.hysteresis_spinbox.setDecimals(2)
        self.hysteresis_spinbox.setMaximum(655.350000000000023)

        self.gridLayout.addWidget(self.hysteresis_spinbox, 1, 1, 1, 1)

        self.edge_comboBox = QComboBox(Param_Pulse)
        self.edge_comboBox.addItem("")
        self.edge_comboBox.addItem("")
        self.edge_comboBox.addItem("")
        self.edge_comboBox.setObjectName(u"edge_comboBox")

        self.gridLayout.addWidget(self.edge_comboBox, 3, 1, 1, 1)


        self.verticalLayout.addLayout(self.gridLayout)

        QWidget.setTabOrder(self.threshold_spinbox, self.hysteresis_spinbox)
        QWidget.setTabOrder(self.hysteresis_spinbox, self.debounce_spinbox)

        self.retranslateUi(Param_Pulse)

        QMetaObject.connectSlotsByName(Param_Pulse)
    # setupUi

    def retranslateUi(self, Param_Pulse):
        self.threshold_spinbox.setSuffix(QCoreApplication.translate("Param_Pulse", u" mV", None))
        self.label_9.setText(QCoreApplication.translate("Param_Pulse", u"Threshold", None))
        self.label_5.setText(QCoreApplication.translate("Param_Pulse", u"Hysteresis", None))
        self.label_10.setText(QCoreApplication.translate("Param_Pulse", u"Debounce time", None))
        self.label_23.setText(QCoreApplication.translate("Param_Pulse", u"Trigger edge", None))
        self.debounce_spinbox.setSuffix(QCoreApplication.translate("Param_Pulse", u" ms", None))
        self.hysteresis_spinbox.setSuffix(QCoreApplication.translate("Param_Pulse", u" mV", None))
        self.edge_comboBox.setItemText(0, QCoreApplication.translate("Param_Pulse", u"rising", None))
        self.edge_comboBox.setItemText(1, QCoreApplication.translate("Param_Pulse", u"falling", None))
        self.edge_comboBox.setItemText(2, QCoreApplication.translate("Param_Pulse", u"rising and falling", None))

        pass
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'main_window.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from  . import main_window_rc

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(1235, 960)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        icon = QIcon()
        icon.addFile(u":/images/ctid-logo.svg", QSize(), QIcon.Normal, QIcon.Off)
        MainWindow.setWindowIcon(icon)
        self.action_preferences = QAction(MainWindow)
        self.action_preferences.setObjectName(u"action_preferences")
        self.actionPreferences = QAction(MainWindow)
        self.actionPreferences.setObjectName(u"actionPreferences")
        self.action_About = QAction(MainWindow)
        self.action_About.setObjectName(u"action_About")
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.verticalLayout_3 = QVBoxLayout(self.centralwidget)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.verticalLayout = QVBoxLayout()
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.common_group = QGroupBox(self.centralwidget)
        self.common_group.setObjectName(u"common_group")
        self.common_group.setEnabled(True)
        sizePolicy1 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.common_group.sizePolicy().hasHeightForWidth())
        self.common_group.setSizePolicy(sizePolicy1)
        self.common_group.setSizeIncrement(QSize(0, 0))
        self.common_group.setAlignment(Qt.AlignLeading|Qt.AlignLeft|Qt.AlignTop)
        self.gridLayout = QGridLayout(self.common_group)
        self.gridLayout.setObjectName(u"gridLayout")
        self.label_23 = QLabel(self.common_group)
        self.label_23.setObjectName(u"label_23")
        font = QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_23.setFont(font)

        self.gridLayout.addWidget(self.label_23, 7, 0, 1, 2)

        self.r_source_spinbox = QSpinBox(self.common_group)
        self.r_source_spinbox.setObjectName(u"r_source_spinbox")
        self.r_source_spinbox.setMaximum(10000000)

        self.gridLayout.addWidget(self.r_source_spinbox, 6, 2, 1, 1)

        self.chip_combo = QComboBox(self.common_group)
        self.chip_combo.addItem("")
        self.chip_combo.addItem("")
        self.chip_combo.addItem("")
        self.chip_combo.setObjectName(u"chip_combo")

        self.gridLayout.addWidget(self.chip_combo, 0, 2, 1, 1)

        self.auto_serial_checkbox = QCheckBox(self.common_group)
        self.auto_serial_checkbox.setObjectName(u"auto_serial_checkbox")

        self.gridLayout.addWidget(self.auto_serial_checkbox, 5, 2, 1, 1)

        self.serial_spinbox = QSpinBox(self.common_group)
        self.serial_spinbox.setObjectName(u"serial_spinbox")
        self.serial_spinbox.setMaximum(16777215)

        self.gridLayout.addWidget(self.serial_spinbox, 4, 2, 1, 1)

        self.sensor_type_combo = QComboBox(self.common_group)
        self.sensor_type_combo.setObjectName(u"sensor_type_combo")
        self.sensor_type_combo.setEditable(False)

        self.gridLayout.addWidget(self.sensor_type_combo, 2, 2, 1, 1)

        self.label_10 = QLabel(self.common_group)
        self.label_10.setObjectName(u"label_10")
        self.label_10.setFont(font)

        self.gridLayout.addWidget(self.label_10, 2, 0, 1, 2)

        self.label_9 = QLabel(self.common_group)
        self.label_9.setObjectName(u"label_9")
        self.label_9.setFont(font)

        self.gridLayout.addWidget(self.label_9, 4, 0, 1, 2)

        self.r_load_spinbox = QSpinBox(self.common_group)
        self.r_load_spinbox.setObjectName(u"r_load_spinbox")
        self.r_load_spinbox.setMaximum(10000000)

        self.gridLayout.addWidget(self.r_load_spinbox, 7, 2, 1, 1)

        self.mfg_combo = QComboBox(self.common_group)
        self.mfg_combo.addItem("")
        self.mfg_combo.setObjectName(u"mfg_combo")

        self.gridLayout.addWidget(self.mfg_combo, 1, 2, 1, 1)

        self.label_11 = QLabel(self.common_group)
        self.label_11.setObjectName(u"label_11")
        self.label_11.setFont(font)

        self.gridLayout.addWidget(self.label_11, 0, 0, 1, 1)

        self.label_3 = QLabel(self.common_group)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setFont(font)

        self.gridLayout.addWidget(self.label_3, 1, 0, 1, 2)

        self.label_24 = QLabel(self.common_group)
        self.label_24.setObjectName(u"label_24")
        self.label_24.setFont(font)

        self.gridLayout.addWidget(self.label_24, 6, 0, 1, 2)

        self.label_5 = QLabel(self.common_group)
        self.label_5.setObjectName(u"label_5")
        self.label_5.setFont(font)

        self.gridLayout.addWidget(self.label_5, 3, 0, 1, 2)

        self.model_lineEdit = QLineEdit(self.common_group)
        self.model_lineEdit.setObjectName(u"model_lineEdit")

        self.gridLayout.addWidget(self.model_lineEdit, 3, 2, 1, 1)


        self.verticalLayout.addWidget(self.common_group)

        self.param_group = QGroupBox(self.centralwidget)
        self.param_group.setObjectName(u"param_group")
        sizePolicy.setHeightForWidth(self.param_group.sizePolicy().hasHeightForWidth())
        self.param_group.setSizePolicy(sizePolicy)
        self.param_group.setAlignment(Qt.AlignLeading|Qt.AlignLeft|Qt.AlignTop)
        self.param_group.setFlat(False)

        self.verticalLayout.addWidget(self.param_group)

        self.horizontalLayout_1 = QHBoxLayout()
        self.horizontalLayout_1.setObjectName(u"horizontalLayout_1")
        self.load_template_btn = QPushButton(self.centralwidget)
        self.load_template_btn.setObjectName(u"load_template_btn")

        self.horizontalLayout_1.addWidget(self.load_template_btn)

        self.save_template_btn = QPushButton(self.centralwidget)
        self.save_template_btn.setObjectName(u"save_template_btn")

        self.horizontalLayout_1.addWidget(self.save_template_btn)


        self.verticalLayout.addLayout(self.horizontalLayout_1)

        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.read_btn = QPushButton(self.centralwidget)
        self.read_btn.setObjectName(u"read_btn")
        sizePolicy2 = QSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.Fixed)
        sizePolicy2.setHorizontalStretch(1)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.read_btn.sizePolicy().hasHeightForWidth())
        self.read_btn.setSizePolicy(sizePolicy2)

        self.horizontalLayout_3.addWidget(self.read_btn)

        self.reprogram_after_cal_btn = QPushButton(self.centralwidget)
        self.reprogram_after_cal_btn.setObjectName(u"reprogram_after_cal_btn")
        sizePolicy3 = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        sizePolicy3.setHorizontalStretch(1)
        sizePolicy3.setVerticalStretch(0)
        sizePolicy3.setHeightForWidth(self.reprogram_after_cal_btn.sizePolicy().hasHeightForWidth())
        self.reprogram_after_cal_btn.setSizePolicy(sizePolicy3)

        self.horizontalLayout_3.addWidget(self.reprogram_after_cal_btn)


        self.verticalLayout.addLayout(self.horizontalLayout_3)

        self.program_btn = QPushButton(self.centralwidget)
        self.program_btn.setObjectName(u"program_btn")
        sizePolicy4 = QSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.Fixed)
        sizePolicy4.setHorizontalStretch(0)
        sizePolicy4.setVerticalStretch(0)
        sizePolicy4.setHeightForWidth(self.program_btn.sizePolicy().hasHeightForWidth())
        self.program_btn.setSizePolicy(sizePolicy4)

        self.verticalLayout.addWidget(self.program_btn)


        self.horizontalLayout.addLayout(self.verticalLayout)

        self.plainTextEdit = QPlainTextEdit(self.centralwidget)
        self.plainTextEdit.setObjectName(u"plainTextEdit")
        sizePolicy5 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        sizePolicy5.setHorizontalStretch(1)
        sizePolicy5.setVerticalStretch(0)
        sizePolicy5.setHeightForWidth(self.plainTextEdit.sizePolicy().hasHeightForWidth())
        self.plainTextEdit.setSizePolicy(sizePolicy5)
        font1 = QFont()
        font1.setFamily(u"Droid Sans Mono")
        self.plainTextEdit.setFont(font1)
        self.plainTextEdit.viewport().setProperty("cursor", QCursor(Qt.IBeamCursor))
        self.plainTextEdit.setFocusPolicy(Qt.NoFocus)
        self.plainTextEdit.setStyleSheet(u"background-color: black; color: white")
        self.plainTextEdit.setUndoRedoEnabled(False)
        self.plainTextEdit.setReadOnly(True)
        self.plainTextEdit.setTextInteractionFlags(Qt.LinksAccessibleByKeyboard|Qt.LinksAccessibleByMouse|Qt.TextBrowserInteraction|Qt.TextSelectableByKeyboard|Qt.TextSelectableByMouse)

        self.horizontalLayout.addWidget(self.plainTextEdit)


        self.verticalLayout_3.addLayout(self.horizontalLayout)

        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(MainWindow)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 1235, 21))
        self.menuEdit = QMenu(self.menubar)
        self.menuEdit.setObjectName(u"menuEdit")
        self.menu_Help = QMenu(self.menubar)
        self.menu_Help.setObjectName(u"menu_Help")
        MainWindow.setMenuBar(self.menubar)
        QWidget.setTabOrder(self.chip_combo, self.mfg_combo)
        QWidget.setTabOrder(self.mfg_combo, self.sensor_type_combo)
        QWidget.setTabOrder(self.sensor_type_combo, self.serial_spinbox)
        QWidget.setTabOrder(self.serial_spinbox, self.auto_serial_checkbox)
        QWidget.setTabOrder(self.auto_serial_checkbox, self.r_source_spinbox)
        QWidget.setTabOrder(self.r_source_spinbox, self.r_load_spinbox)
        QWidget.setTabOrder(self.r_load_spinbox, self.load_template_btn)
        QWidget.setTabOrder(self.load_template_btn, self.save_template_btn)

        self.menubar.addAction(self.menuEdit.menuAction())
        self.menubar.addAction(self.menu_Help.menuAction())
        self.menuEdit.addAction(self.actionPreferences)
        self.menu_Help.addAction(self.action_About)

        self.retranslateUi(MainWindow)

        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"CTid\u00ae Programmer", None))
        self.action_preferences.setText(QCoreApplication.translate("MainWindow", u"Preferences", None))
        self.actionPreferences.setText(QCoreApplication.translate("MainWindow", u"Preferences", None))
#if QT_CONFIG(shortcut)
        self.actionPreferences.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+P", None))
#endif // QT_CONFIG(shortcut)
        self.action_About.setText(QCoreApplication.translate("MainWindow", u"&About", None))
        self.common_group.setTitle(QCoreApplication.translate("MainWindow", u"Common Information", None))
        self.label_23.setText(QCoreApplication.translate("MainWindow", u"Test Load Impedance", None))
        self.r_source_spinbox.setSuffix(QCoreApplication.translate("MainWindow", u" \u03a9", None))
        self.chip_combo.setItemText(0, QCoreApplication.translate("MainWindow", u"Auto Detect", None))
        self.chip_combo.setItemText(1, QCoreApplication.translate("MainWindow", u"ATtiny9", None))
        self.chip_combo.setItemText(2, QCoreApplication.translate("MainWindow", u"ATtiny10", None))

        self.auto_serial_checkbox.setText(QCoreApplication.translate("MainWindow", u"Automatically Select Serial Number", None))
        self.label_10.setText(QCoreApplication.translate("MainWindow", u"Sensor Type", None))
        self.label_9.setText(QCoreApplication.translate("MainWindow", u"Serial Number", None))
        self.r_load_spinbox.setSuffix(QCoreApplication.translate("MainWindow", u" \u03a9", None))
        self.mfg_combo.setItemText(0, "")

        self.label_11.setText(QCoreApplication.translate("MainWindow", u"Microchip", None))
        self.label_3.setText(QCoreApplication.translate("MainWindow", u"Manufacturer", None))
        self.label_24.setText(QCoreApplication.translate("MainWindow", u"Sensor Impedance", None))
        self.label_5.setText(QCoreApplication.translate("MainWindow", u"Model Name", None))
        self.param_group.setTitle(QCoreApplication.translate("MainWindow", u"Sensor Parameters", None))
        self.load_template_btn.setText(QCoreApplication.translate("MainWindow", u"Load Template", None))
        self.save_template_btn.setText(QCoreApplication.translate("MainWindow", u"Save as Template", None))
        self.read_btn.setText(QCoreApplication.translate("MainWindow", u"Read", None))
        self.reprogram_after_cal_btn.setText(QCoreApplication.translate("MainWindow", u"Reprogram after Cal", None))
        self.program_btn.setText(QCoreApplication.translate("MainWindow", u"Program", None))
        self.menuEdit.setTitle(QCoreApplication.translate("MainWindow", u"&Edit", None))
        self.menu_Help.setTitle(QCoreApplication.translate("MainWindow", u"&Help", None))
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'param_volt.ui'
##
## Created by: Qt User Interface Compiler version 5.15.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_Param_Volt(object):
    def setupUi(self, Param_Volt):
        if not Param_Volt.objectName():
            Param_Volt.setObjectName(u"Param_Volt")
        Param_Volt.resize(317, 177)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Param_Volt.sizePolicy().hasHeightForWidth())
        Param_Volt.setSizePolicy(sizePolicy)
        self.gridLayout = QGridLayout(Param_Volt)
        self.gridLayout.setObjectName(u"gridLayout")
        self.gridLayout.setContentsMargins(-1, 24, -1, -1)
        self.label_9 = QLabel(Param_Volt)
        self.label_9.setObjectName(u"label_9")
        font = QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_9.setFont(font)

        self.gridLayout.addWidget(self.label_9, 0, 0, 1, 1)

        self.scale_spinbox = QDoubleSpinBox(Param_Volt)
        self.scale_spinbox.setObjectName(u"scale_spinbox")
        sizePolicy1 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.scale_spinbox.sizePolicy().hasHeightForWidth())
        self.scale_spinbox.setSizePolicy(sizePolicy1)
        self.scale_spinbox.setDecimals(3)
        self.scale_spinbox.setMinimum(-1000000.000000000000000)
        self.scale_spinbox.setMaximum(1000000.000000000000000)

        self.gridLayout.addWidget(self.scale_spinbox, 0, 1, 1, 1)

        self.label_5 = QLabel(Param_Volt)
        self.label_5.setObjectName(u"label_5")
        self.label_5.setFont(font)

        self.gridLayout.addWidget(self.label_5, 1, 0, 1, 1)

        self.offset_spinbox = QDoubleSpinBox(Param_Volt)
        self.offset_spinbox.setObjectName(u"offset_spinbox")
        self.offset_spinbox.setDecimals(3)
        self.offset_spinbox.setMinimum(-1000000.000000000000000)
        self.offset_spinbox.setMaximum(1000000.000000000000000)

        self.gridLayout.addWidget(self.offset_spinbox, 1, 1, 1, 1)

        self.label_10 = QLabel(Param_Volt)
        self.label_10.setObjectName(u"label_10")
        self.label_10.setFont(font)

        self.gridLayout.addWidget(self.label_10, 2, 0, 1, 1)

        self.delay_spinbox = QDoubleSpinBox(Param_Volt)
        self.delay_spinbox.setObjectName(u"delay_spinbox")
        self.delay_spinbox.setDecimals(2)
        self.delay_spinbox.setMinimum(-327.680000000000007)
        self.delay_spinbox.setMaximum(327.670000000000016)

        self.gridLayout.addWidget(self.delay_spinbox, 2, 1, 1, 1)

        self.verticalSpacer = QSpacerItem(20, 42, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.gridLayout.addItem(self.verticalSpacer, 3, 1, 1, 1)

        QWidget.setTabOrder(self.scale_spinbox, self.offset_spinbox)
        QWidget.setTabOrder(self.offset_spinbox, self.delay_spinbox)

        self.retranslateUi(Param_Volt)

        QMetaObject.connectSlotsByName(Param_Volt)
    # setupUi

    def retranslateUi(self, Param_Volt):
        self.label_9.setText(QCoreApplication.translate("Param_Volt", u"Scale", None))
        self.scale_spinbox.setSuffix(QCoreApplication.translate("Param_Volt", u" V/V", None))
        self.label_5.setText(QCoreApplication.translate("Param_Volt", u"Offset", None))
        self.offset_spinbox.setSuffix(QCoreApplication.translate("Param_Volt", u" V", None))
        self.label_10.setText(QCoreApplication.translate("Param_Volt", u"Delay", None))
        self.delay_spinbox.setSuffix(QCoreApplication.translate("Param_Volt", u" \u03bcs", None))
        pass
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'egauge_api_credentials_dialog.ui'
##
## Created by: Qt User Interface Compiler version 5.15.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_Credentials_Dialog(object):
    def setupUi(self, Credentials_Dialog):
        if not Credentials_Dialog.objectName():
            Credentials_Dialog.setObjectName(u"Credentials_Dialog")
        Credentials_Dialog.resize(307, 174)
        self.gridLayout = QGridLayout(Credentials_Dialog)
        self.gridLayout.setObjectName(u"gridLayout")
        self.buttonBox = QDialogButtonBox(Credentials_Dialog)
        self.buttonBox.setObjectName(u"buttonBox")
        self.buttonBox.setOrientation(Qt.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.Cancel|QDialogButtonBox.Ok)

        self.gridLayout.addWidget(self.buttonBox, 3, 0, 1, 2)

        self.label = QLabel(Credentials_Dialog)
        self.label.setObjectName(u"label")

        self.gridLayout.addWidget(self.label, 2, 0, 1, 1)

        self.label_3 = QLabel(Credentials_Dialog)
        self.label_3.setObjectName(u"label_3")

        self.gridLayout.addWidget(self.label_3, 1, 0, 1, 1)

        self.username_lineEdit = QLineEdit(Credentials_Dialog)
        self.username_lineEdit.setObjectName(u"username_lineEdit")

        self.gridLayout.addWidget(self.username_lineEdit, 1, 1, 1, 1)

        self.password_lineEdit = QLineEdit(Credentials_Dialog)
        self.password_lineEdit.setObjectName(u"password_lineEdit")
        self.password_lineEdit.setInputMask(u"")
        self.password_lineEdit.setText(u"")
        self.password_lineEdit.setEchoMode(QLineEdit.Password)

        self.gridLayout.addWidget(self.password_lineEdit, 2, 1, 1, 1)

        self.prompt_label = QLabel(Credentials_Dialog)
        self.prompt_label.setObjectName(u"prompt_label")
        self.prompt_label.setWordWrap(True)

        self.gridLayout.addWidget(self.prompt_label, 0, 0, 1, 2)


        self.retranslateUi(Credentials_Dialog)
        self.buttonBox.accepted.connect(Credentials_Dialog.accept)
        self.buttonBox.rejected.connect(Credentials_Dialog.reject)

        QMetaObject.connectSlotsByName(Credentials_Dialog)
    # setupUi

    def retranslateUi(self, Credentials_Dialog):
        Credentials_Dialog.setWindowTitle(QCoreApplication.translate("Credentials_Dialog", u"eGauge API Login", None))
        self.label.setText(QCoreApplication.translate("Credentials_Dialog", u"Password", None))
        self.label_3.setText(QCoreApplication.translate("Credentials_Dialog", u"Username", None))
        self.prompt_label.setText(QCoreApplication.translate("Credentials_Dialog", u"Please enter your eGuard username and password.", None))
    # retranslateUi




import os
import setuptools

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setuptools.setup(
    name='CTid-programmer',
    version='1.3.8',
    packages=setuptools.find_packages(),
    package_data={
        '': ['resources/code/*.hex']
    },
    install_requires=['egauge-python>=0.5.2,!=0.21',
                      'importlib_resources', 'wheel'],
    entry_points={
        'console_scripts': [
            'CTid-programmer = ctid_programmer.programmer:main'
        ]
    },
    license='MIT License',  # example license
    description='A graphical user interface for programming CTid sensors.',
    long_description=README,
    long_description_content_type='text/x-rst',
    url='https://bitbucket.org/egauge/CTid-programmer/',
    author='David Mosberger-Tang',
    author_email='davidm@egauge.net',
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Manufacturing',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Topic :: System :: Hardware'
    ],
)



#
# Copyright (c) 2020 eGauge Systems LLC
#       1644 Conestoga St, Suite 2
#       Boulder, CO 80301
#       voice: 720-545-9767
#       email: davidm@egauge.net
#
# All rights reserved.
#
# MIT License
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
import json
import os

class Manager:
    def __init__(self, state_directory_path):
        self.path = os.path.join(state_directory_path, 'templates.json')
        self.templates = {}
        try:
            with open(self.path, 'r') as f:
                self.templates = json.load(f)
        except FileNotFoundError:
            try:
                # try to create a new, empty file:
                with open(self.path, 'w') as f:
                    json.dump({}, f)
            except IOError:
                pass
        except IOError:
            pass

    def _save_state(self):
        with open(self.path, 'w') as f:
            json.dump(self.templates, f, sort_keys=True, indent=2)

    def items(self):
        '''Return a iterator of {name: template} items.'''
        return self.templates.items()

    def load(self, name):
        return self.templates[name]

    def save(self, template, name):
        self.templates[name] = template
        self._save_state()

    def remove(self, name):
        del self.templates[name]
        self._save_state()

    def may_save(self):
        return os.access(self.path, os.W_OK)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'template_dialog.ui'
##
## Created by: Qt User Interface Compiler version 5.15.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_Template_Dialog(object):
    def setupUi(self, Template_Dialog):
        if not Template_Dialog.objectName():
            Template_Dialog.setObjectName(u"Template_Dialog")
        Template_Dialog.setWindowModality(Qt.ApplicationModal)
        Template_Dialog.resize(809, 530)
        Template_Dialog.setModal(True)
        self.horizontalLayout_2 = QHBoxLayout(Template_Dialog)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.template_name_frame = QFrame(Template_Dialog)
        self.template_name_frame.setObjectName(u"template_name_frame")
        self.template_name_frame.setFrameShape(QFrame.StyledPanel)
        self.template_name_frame.setFrameShadow(QFrame.Raised)
        self.horizontalLayout = QHBoxLayout(self.template_name_frame)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.label_2 = QLabel(self.template_name_frame)
        self.label_2.setObjectName(u"label_2")

        self.horizontalLayout.addWidget(self.label_2)

        self.lineEdit = QLineEdit(self.template_name_frame)
        self.lineEdit.setObjectName(u"lineEdit")

        self.horizontalLayout.addWidget(self.lineEdit)


        self.verticalLayout_2.addWidget(self.template_name_frame)

        self.existing_templates_frame = QFrame(Template_Dialog)
        self.existing_templates_frame.setObjectName(u"existing_templates_frame")
        self.existing_templates_frame.setFrameShape(QFrame.StyledPanel)
        self.existing_templates_frame.setFrameShadow(QFrame.Raised)
        self.verticalLayout = QVBoxLayout(self.existing_templates_frame)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.label = QLabel(self.existing_templates_frame)
        self.label.setObjectName(u"label")

        self.verticalLayout.addWidget(self.label)

        self.listWidget = QListWidget(self.existing_templates_frame)
        self.listWidget.setObjectName(u"listWidget")
        self.listWidget.setContextMenuPolicy(Qt.ActionsContextMenu)

        self.verticalLayout.addWidget(self.listWidget)


        self.verticalLayout_2.addWidget(self.existing_templates_frame)


        self.horizontalLayout_2.addLayout(self.verticalLayout_2)

        self.buttonBox = QDialogButtonBox(Template_Dialog)
        self.buttonBox.setObjectName(u"buttonBox")
        self.buttonBox.setOrientation(Qt.Vertical)
        self.buttonBox.setStandardButtons(QDialogButtonBox.Cancel|QDialogButtonBox.Open|QDialogButtonBox.Save)

        self.horizontalLayout_2.addWidget(self.buttonBox)


        self.retranslateUi(Template_Dialog)
        self.buttonBox.accepted.connect(Template_Dialog.accept)
        self.buttonBox.rejected.connect(Template_Dialog.reject)

        QMetaObject.connectSlotsByName(Template_Dialog)
    # setupUi

    def retranslateUi(self, Template_Dialog):
        Template_Dialog.setWindowTitle(QCoreApplication.translate("Template_Dialog", u"CTid Templates", None))
        self.label_2.setText(QCoreApplication.translate("Template_Dialog", u"New template name:", None))
        self.label.setText(QCoreApplication.translate("Template_Dialog", u"Existing Templates (right click to rename or delete):", None))
    # retranslateUi

